@@ -17,32 +17,28 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "bytestream-factory.h"
 
-#include <stdlib.h>
 #include <string.h>
-#include <time.h>
-
-#include <glib.h>
+#include <stdlib.h>
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-
 #include <loudmouth/loudmouth.h>
+#include <telepathy-glib/interfaces.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_BYTESTREAM
 
-#include <telepathy-glib/interfaces.h>
-#include "debug.h"
-#include "gabble-connection.h"
-#include "bytestream-iface.h"
 #include "bytestream-ibb.h"
+#include "bytestream-iface.h"
 #include "bytestream-muc.h"
+#include "connection.h"
+#include "debug.h"
 #include "namespaces.h"
-#include "util.h"
 #include "presence-cache.h"
-
 #include "private-tubes-factory.h"
+#include "util.h"
 
 G_DEFINE_TYPE (GabbleBytestreamFactory, gabble_bytestream_factory,
     G_TYPE_OBJECT);
@@ -107,7 +103,6 @@ bytestream_id_free (gpointer v)
   g_slice_free (BytestreamIdentifier, bsid);
 }
 
-typedef struct _GabbleBytestreamFactoryPrivate GabbleBytestreamFactoryPrivate;
 struct _GabbleBytestreamFactoryPrivate
 {
   GabbleConnection *conn;
@@ -129,8 +124,7 @@ struct _GabbleBytestreamFactoryPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE(obj) \
-    ((GabbleBytestreamFactoryPrivate *) obj->priv)
+#define GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE(obj) ((obj)->priv)
 
 static LmHandlerResult
 bytestream_factory_msg_data_cb (LmMessageHandler *handler,
