@@ -17,29 +17,24 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "bytestream-ibb.h"
 
-#include <stdlib.h>
 #include <string.h>
-#include <time.h>
-
-#include <glib.h>
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-
 #include <loudmouth/loudmouth.h>
-
 #include <telepathy-glib/interfaces.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_BYTESTREAM
 
-#include "bytestream-iface.h"
 #include "base64.h"
 #include "bytestream-factory.h"
+#include "bytestream-iface.h"
+#include "connection.h"
 #include "debug.h"
 #include "disco.h"
-#include "gabble-connection.h"
 #include "gabble-signals-marshal.h"
 #include "namespaces.h"
 #include "util.h"
@@ -78,7 +73,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef struct _GabbleBytestreamIBBPrivate GabbleBytestreamIBBPrivate;
 struct _GabbleBytestreamIBBPrivate
 {
   GabbleConnection *conn;
@@ -95,8 +89,7 @@ struct _GabbleBytestreamIBBPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_BYTESTREAM_IBB_GET_PRIVATE(obj) \
-    ((GabbleBytestreamIBBPrivate *) obj->priv)
+#define GABBLE_BYTESTREAM_IBB_GET_PRIVATE(obj) ((obj)->priv)
 
 static void
 gabble_bytestream_ibb_init (GabbleBytestreamIBB *self)
