@@ -17,11 +17,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "bytestream-iface.h"
 
-#include <glib.h>
-
-#include "gabble-connection.h"
+#include "connection.h"
 
 gboolean
 gabble_bytestream_iface_initiate (GabbleBytestreamIface *self)
