@@ -18,11 +18,18 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "conn-aliasing.h"
 
+#include <telepathy-glib/contacts-mixin.h>
+#include <telepathy-glib/gtypes.h>
+#include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/svc-connection.h>
 
-#include "gabble-connection.h"
+#define DEBUG_FLAG GABBLE_DEBUG_CONNECTION
+
+#include "connection.h"
+#include "debug.h"
 #include "namespaces.h"
 #include "presence-cache.h"
 #include "pubsub.h"
@@ -31,13 +38,6 @@
 #include "util.h"
 #include "vcard-manager.h"
 
-#define DEBUG_FLAG GABBLE_DEBUG_CONNECTION
-
-#include "debug.h"
-
-#define GABBLE_TP_ALIAS_PAIR_TYPE (dbus_g_type_get_struct ("GValueArray", \
-      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID))
-
 static void gabble_conn_aliasing_pep_nick_reply_handler (
     GabbleConnection *conn, LmMessage *msg, TpHandle handle);
 static GQuark gabble_conn_aliasing_pep_alias_quark (void);
@@ -214,6 +214,52 @@ _cache_negatively (GabbleConnection *self,
       gabble_conn_aliasing_pep_alias_quark (), (gchar *) NO_ALIAS, NULL);
 }
 
+/* Cache pep if successful */
+static void
+aliases_request_cache_pep (GabbleConnection *self,
+                           LmMessage *msg,
+                           TpHandle handle,
+                           GError *error)
+{
+  if (error != NULL)
+    {
+      DEBUG ("Error getting alias from PEP: %s", error->message);
+      _cache_negatively (self, handle);
+    }
+  else if (lm_message_get_sub_type (msg) != LM_MESSAGE_SUB_TYPE_RESULT)
+    {
+      NODE_DEBUG (msg->node, "Error getting alias from PEP");
+      _cache_negatively (self, handle);
+    }
+  else
+    {
+      /* Try to extract an alias, caching it if necessary. */
+      gabble_conn_aliasing_pep_nick_reply_handler (self, msg, handle);
+    }
+
+}
+
+static void
+aliases_request_basic_pep_cb (GabbleConnection *self,
+                              LmMessage *msg,
+                              gpointer user_data,
+                              GError *error)
+{
+  GabbleConnectionAliasSource source = GABBLE_CONNECTION_ALIAS_NONE;
+  TpHandle handle = GPOINTER_TO_INT (user_data);
+
+  aliases_request_cache_pep (self, msg, handle, error);
+
+  source = _gabble_connection_get_cached_alias (self, handle, NULL);
+
+  if (source < GABBLE_CONNECTION_ALIAS_FROM_VCARD &&
+      !gabble_vcard_manager_has_cached_alias (self->vcard_manager, handle))
+    {
+      /* no alias in PEP, get the vcard */
+      gabble_vcard_manager_request (self->vcard_manager, handle, 0,
+        NULL, NULL, G_OBJECT (self));
+    }
+}
 
 static void
 aliases_request_pep_cb (GabbleConnection *self,
@@ -232,21 +278,7 @@ aliases_request_pep_cb (GabbleConnection *self,
   aliases_request->pep_requests[index] = NULL;
   g_slice_free (AliasRequest, alias_request);
 
-  if (error != NULL)
-    {
-      DEBUG ("Error getting alias from PEP: %s", error->message);
-      _cache_negatively (self, handle);
-    }
-  else if (lm_message_get_sub_type (msg) != LM_MESSAGE_SUB_TYPE_RESULT)
-    {
-      NODE_DEBUG (msg->node, "Error getting alias from PEP");
-      _cache_negatively (self, handle);
-    }
-  else
-    {
-      /* Try to extract an alias, caching it if necessary. */
-      gabble_conn_aliasing_pep_nick_reply_handler (self, msg, handle);
-    }
+  aliases_request_cache_pep (self, msg, handle, error);
 
   source = _gabble_connection_get_cached_alias (aliases_request->conn,
       handle, &alias);
@@ -276,6 +308,32 @@ aliases_request_pep_cb (GabbleConnection *self,
 }
 
 
+static GabbleRequestPipelineItem *
+gabble_do_pep_request (GabbleConnection *self,
+                       TpHandle handle,
+                       TpHandleRepoIface *contact_handles,
+                       GabbleRequestPipelineCb callback,
+                       gpointer user_data)
+{
+  LmMessage *msg;
+  GabbleRequestPipelineItem *pep_request;
+
+  msg = lm_message_build (tp_handle_inspect (contact_handles, handle),
+      LM_MESSAGE_TYPE_IQ,
+      '(', "pubsub", "",
+        '@', "xmlns", NS_PUBSUB,
+        '(', "items", "",
+          '@', "node", NS_NICK,
+        ')',
+      ')',
+      NULL);
+   pep_request = gabble_request_pipeline_enqueue (self->req_pipeline,
+      msg, 0, callback, user_data);
+   lm_message_unref (msg);
+
+   return pep_request;
+}
+
 /**
  * gabble_connection_request_aliases
  *
@@ -291,7 +349,7 @@ gabble_connection_request_aliases (TpSvcConnectionInterfaceAliasing *iface,
                                    DBusGMethodInvocation *context)
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
-  TpBaseConnection *base = (TpBaseConnection *)self;
+  TpBaseConnection *base = (TpBaseConnection *) self;
   TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
   guint i;
@@ -337,30 +395,16 @@ gabble_connection_request_aliases (TpSvcConnectionInterfaceAliasing *iface,
             * current ejabberd PEP implementation doesn't seem to give
             * us notifications of the initial state. */
           AliasRequest *data = g_slice_new (AliasRequest);
-          LmMessage *msg;
-          GabbleRequestPipelineItem *pep_request;
 
           g_free (alias);
 
-          msg = lm_message_build (tp_handle_inspect (contact_handles, handle),
-              LM_MESSAGE_TYPE_IQ,
-              '(', "pubsub", "",
-                '@', "xmlns", NS_PUBSUB,
-                '(', "items", "",
-                  '@', "node", NS_NICK,
-                ')',
-              ')',
-              NULL);
-
           data->aliases_request = request;
           data->index = i;
 
-          pep_request = gabble_request_pipeline_enqueue (self->req_pipeline,
-              msg, 0, aliases_request_pep_cb, data);
-          request->pep_requests[i] = pep_request;
           request->pending_pep_requests++;
+          request->pep_requests[i] = gabble_do_pep_request (self,
+              handle, contact_handles, aliases_request_pep_cb, data);
 
-          lm_message_unref (msg);
         }
       else
         {
@@ -422,7 +466,7 @@ setaliases_foreach (gpointer key, gpointer value, gpointer user_data)
   TpHandle handle = GPOINTER_TO_INT (key);
   gchar *alias = (gchar *) value;
   GError *error = NULL;
-  TpBaseConnection *base = (TpBaseConnection *)data->conn;
+  TpBaseConnection *base = (TpBaseConnection *) data->conn;
   TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
 
@@ -500,7 +544,7 @@ gabble_connection_set_aliases (TpSvcConnectionInterfaceAliasing *iface,
                                DBusGMethodInvocation *context)
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
-  TpBaseConnection *base = (TpBaseConnection *)self;
+  TpBaseConnection *base = (TpBaseConnection *) self;
   GError *error = NULL;
   struct _i_hate_g_hash_table_foreach data = { NULL, NULL, TRUE };
 
@@ -698,9 +742,9 @@ gabble_conn_aliasing_nickname_updated (GObject *object,
       goto OUT;
     }
 
-  g_value_init (&entry, GABBLE_TP_ALIAS_PAIR_TYPE);
+  g_value_init (&entry, TP_STRUCT_TYPE_ALIAS_PAIR);
   g_value_take_boxed (&entry, dbus_g_type_specialized_construct
-      (GABBLE_TP_ALIAS_PAIR_TYPE));
+      (TP_STRUCT_TYPE_ALIAS_PAIR));
 
   dbus_g_type_struct_set (&entry,
       0, handle,
@@ -726,7 +770,7 @@ _gabble_connection_get_cached_alias (GabbleConnection *conn,
                                      TpHandle handle,
                                      gchar **alias)
 {
-  TpBaseConnection *base = (TpBaseConnection *)conn;
+  TpBaseConnection *base = (TpBaseConnection *) conn;
   TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
   GabbleConnectionAliasSource ret = GABBLE_CONNECTION_ALIAS_NONE;
@@ -847,6 +891,121 @@ OUT:
   return ret;
 }
 
+static void
+maybe_request_vcard (GabbleConnection *self, TpHandle handle,
+  GabbleConnectionAliasSource source)
+{
+  /* If the source wasn't good enough then do a request */
+  if (source < GABBLE_CONNECTION_ALIAS_FROM_VCARD &&
+      !gabble_vcard_manager_has_cached_alias (self->vcard_manager, handle))
+    {
+      if (self->features & GABBLE_CONNECTION_FEATURES_PEP)
+        {
+          TpBaseConnection *base = (TpBaseConnection *) self;
+          TpHandleRepoIface *contact_handles =
+            tp_base_connection_get_handles (base, TP_HANDLE_TYPE_CONTACT);
+
+          gabble_do_pep_request (self, handle, contact_handles,
+            aliases_request_basic_pep_cb, GINT_TO_POINTER (handle));
+        }
+      else
+        {
+          gabble_vcard_manager_request (self->vcard_manager,
+             handle, 0, NULL, NULL, G_OBJECT (self));
+        }
+    }
+}
+
+static void
+conn_aliasing_fill_contact_attributes (GObject *obj,
+    const GArray *contacts, GHashTable *attributes_hash)
+{
+  guint i;
+  GabbleConnection *self = GABBLE_CONNECTION(obj);
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      TpHandle handle = g_array_index (contacts, TpHandle, i);
+      GabbleConnectionAliasSource source;
+      gchar *alias;
+      GValue *val = tp_g_value_slice_new (G_TYPE_STRING);
+
+      source = _gabble_connection_get_cached_alias (self, handle, &alias);
+      g_assert (alias != NULL);
+
+      g_value_take_string (val, alias);
+
+      tp_contacts_mixin_set_contact_attribute (attributes_hash,
+        handle, TP_IFACE_CONNECTION_INTERFACE_ALIASING"/alias",
+        val);
+
+      maybe_request_vcard (self, handle, source);
+    }
+}
+
+/**
+ * gabble_connection_get_aliases
+ *
+ * Implements D-Bus method GetAliases
+ * on interface org.freedesktop.Telepathy.Connection.Interface.Aliasing
+ *
+ * @context: The D-Bus invocation context to use to return values
+ *           or throw an error.
+ */
+static void
+gabble_connection_get_aliases (TpSvcConnectionInterfaceAliasing *iface,
+                               const GArray *contacts,
+                               DBusGMethodInvocation *context)
+{
+  GabbleConnection *self = GABBLE_CONNECTION (iface);
+  TpBaseConnection *base = (TpBaseConnection *) self;
+  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
+      TP_HANDLE_TYPE_CONTACT);
+  GHashTable *result = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+    NULL, g_free);
+  GError *error = NULL;
+  guint i;
+
+  g_assert (GABBLE_IS_CONNECTION (self));
+
+  TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, context);
+
+  if (!tp_handles_are_valid (contact_handles, contacts, FALSE, &error))
+    {
+      dbus_g_method_return_error (context, error);
+      g_error_free (error);
+      return;
+    }
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      TpHandle handle = g_array_index (contacts, TpHandle, i);
+      GabbleConnectionAliasSource source;
+      gchar *alias;
+
+      source = _gabble_connection_get_cached_alias (self, handle, &alias);
+      g_assert (alias != NULL);
+
+      g_hash_table_insert (result, GUINT_TO_POINTER (handle), alias);
+
+      maybe_request_vcard (self, handle, source);
+    }
+
+  tp_svc_connection_interface_aliasing_return_from_get_aliases (context,
+    result);
+
+  g_hash_table_destroy (result);
+}
+
+
+
+void
+conn_aliasing_init (GabbleConnection *conn)
+{
+  tp_contacts_mixin_add_contact_attributes_iface (G_OBJECT (conn),
+    TP_IFACE_CONNECTION_INTERFACE_ALIASING,
+    conn_aliasing_fill_contact_attributes);
+}
 
 void
 conn_aliasing_iface_init (gpointer g_iface, gpointer iface_data)
@@ -858,6 +1017,7 @@ conn_aliasing_iface_init (gpointer g_iface, gpointer iface_data)
     klass, gabble_connection_##x)
   IMPLEMENT(get_alias_flags);
   IMPLEMENT(request_aliases);
+  IMPLEMENT(get_aliases);
   IMPLEMENT(set_aliases);
 #undef IMPLEMENT
 }
