@@ -27,6 +27,8 @@
 #include <loudmouth/loudmouth.h>
 
 #include <telepathy-glib/svc-connection.h>
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/contacts-mixin.h>
 
 #include "base64.h"
 #include "presence.h"
@@ -47,7 +49,7 @@ update_own_avatar_sha1 (GabbleConnection *conn,
                         const gchar *sha1,
                         GError **out_error)
 {
-  TpBaseConnection *base = (TpBaseConnection *)conn;
+  TpBaseConnection *base = (TpBaseConnection *) conn;
   GError *error = NULL;
 
   if (!tp_strdiff (sha1, conn->self_presence->avatar_sha1))
@@ -79,7 +81,7 @@ connection_avatar_update_cb (GabblePresenceCache *cache,
                              gpointer user_data)
 {
   GabbleConnection *conn = GABBLE_CONNECTION (user_data);
-  TpBaseConnection *base = (TpBaseConnection *)conn;
+  TpBaseConnection *base = (TpBaseConnection *) conn;
   GabblePresence *presence;
 
   presence = gabble_presence_cache_get (conn->presence_cache, handle);
@@ -184,7 +186,7 @@ gabble_connection_get_avatar_tokens (TpSvcConnectionInterfaceAvatars *iface,
                                      DBusGMethodInvocation *invocation)
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
-  TpBaseConnection *base = (TpBaseConnection *)self;
+  TpBaseConnection *base = (TpBaseConnection *) self;
   gboolean wait_for_self_avatar = FALSE;
   gboolean have_self_avatar;
   guint i, my_index = 0;
@@ -312,7 +314,7 @@ gabble_connection_get_known_avatar_tokens (TpSvcConnectionInterfaceAvatars *ifac
                                            DBusGMethodInvocation *invocation)
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
-  TpBaseConnection *base = (TpBaseConnection *)self;
+  TpBaseConnection *base = (TpBaseConnection *) self;
   gboolean wait_for_self_avatar = FALSE;
   gboolean have_self_avatar;
   guint i;
@@ -564,7 +566,7 @@ gabble_connection_request_avatar (TpSvcConnectionInterfaceAvatars *iface,
                                   DBusGMethodInvocation *context)
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
-  TpBaseConnection *base = (TpBaseConnection *)self;
+  TpBaseConnection *base = (TpBaseConnection *) self;
   TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
   GError *err = NULL;
@@ -721,7 +723,7 @@ _set_avatar_cb2 (GabbleVCardManager *manager,
                  gpointer user_data)
 {
   struct _set_avatar_ctx *ctx = (struct _set_avatar_ctx *) user_data;
-  TpBaseConnection *base = (TpBaseConnection *)ctx->conn;
+  TpBaseConnection *base = (TpBaseConnection *) ctx->conn;
 
   if (NULL == vcard)
     {
@@ -777,7 +779,7 @@ gabble_connection_set_avatar (TpSvcConnectionInterfaceAvatars *iface,
                               DBusGMethodInvocation *context)
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
-  TpBaseConnection *base = (TpBaseConnection *)self;
+  TpBaseConnection *base = (TpBaseConnection *) self;
   struct _set_avatar_ctx *ctx;
   gchar *value = NULL;
   gchar *base64;
@@ -820,6 +822,39 @@ gabble_connection_clear_avatar (TpSvcConnectionInterfaceAvatars *iface,
   gabble_connection_set_avatar (iface, NULL, NULL, context);
 }
 
+static void
+conn_avatars_fill_contact_attributes (GObject *obj,
+    const GArray *contacts, GHashTable *attributes_hash)
+{
+  guint i;
+  GabbleConnection *self = GABBLE_CONNECTION(obj);
+  TpBaseConnection *base = (TpBaseConnection *) self;
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      TpHandle handle = g_array_index (contacts, guint, i);
+      GabblePresence *presence = NULL;
+
+      if (base->self_handle == handle)
+        presence = self->self_presence;
+      else
+        presence = gabble_presence_cache_get (self->presence_cache, handle);
+
+      if (NULL != presence)
+        {
+          GValue *val = tp_g_value_slice_new (G_TYPE_STRING);
+
+          if (NULL != presence->avatar_sha1)
+            g_value_set_string (val, presence->avatar_sha1);
+          else
+            g_value_set_string (val, "");
+
+          tp_contacts_mixin_set_contact_attribute (attributes_hash, handle,
+            TP_IFACE_CONNECTION_INTERFACE_AVATARS"/token", val);
+        }
+    }
+}
+
 
 void
 conn_avatars_init (GabbleConnection *conn)
@@ -828,6 +863,10 @@ conn_avatars_init (GabbleConnection *conn)
       (connection_got_self_initial_avatar_cb), conn);
   g_signal_connect (conn->presence_cache, "avatar-update", G_CALLBACK
       (connection_avatar_update_cb), conn);
+
+  tp_contacts_mixin_add_contact_attributes_iface (G_OBJECT (conn),
+      TP_IFACE_CONNECTION_INTERFACE_AVATARS,
+          conn_avatars_fill_contact_attributes);
 }
 
 
