@@ -21,22 +21,23 @@
  * -- LET'S DISCO!!!  \o/ \o_ _o/ /\o/\ _/o/- -\o\_ --
  */
 
+#include "config.h"
 #include "disco.h"
 
+#include <string.h>
+
 #define DBUS_API_SUBJECT_TO_CHANGE
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-#include <stdlib.h>
-#include <string.h>
+#include <telepathy-glib/dbus.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_DISCO
 
+#include "connection.h"
 #include "debug.h"
-#include "gabble-connection.h"
-#include "gabble-error.h"
+#include "error.h"
 #include "namespaces.h"
-#include <telepathy-glib/dbus.h>
 #include "util.h"
 
 #define DEFAULT_REQUEST_TIMEOUT 20000
@@ -51,7 +52,6 @@ enum
 
 G_DEFINE_TYPE(GabbleDisco, gabble_disco, G_TYPE_OBJECT);
 
-typedef struct _GabbleDiscoPrivate GabbleDiscoPrivate;
 struct _GabbleDiscoPrivate
 {
   GabbleConnection *connection;
@@ -82,7 +82,7 @@ gabble_disco_error_quark (void)
   return quark;
 }
 
-#define GABBLE_DISCO_GET_PRIVATE(o)     ((GabbleDiscoPrivate*)((o)->priv))
+#define GABBLE_DISCO_GET_PRIVATE(o) ((o)->priv)
 
 static void
 gabble_disco_init (GabbleDisco *obj)
@@ -292,7 +292,7 @@ delete_request (GabbleDiscoRequest *request)
 static gboolean
 timeout_request (gpointer data)
 {
-  GabbleDiscoRequest *request = (GabbleDiscoRequest*) data;
+  GabbleDiscoRequest *request = (GabbleDiscoRequest *) data;
   GabbleDisco *disco;
   GError *err /* doesn't need initializing */;
   g_return_val_if_fail (data != NULL, FALSE);
@@ -364,7 +364,7 @@ static LmHandlerResult
 request_reply_cb (GabbleConnection *conn, LmMessage *sent_msg,
                   LmMessage *reply_msg, GObject *object, gpointer user_data)
 {
-  GabbleDiscoRequest *request = (GabbleDiscoRequest*) user_data;
+  GabbleDiscoRequest *request = (GabbleDiscoRequest *) user_data;
   GabbleDisco *disco = GABBLE_DISCO (object);
   GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (disco);
   LmMessageNode *query_node;
