@@ -17,30 +17,26 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "im-factory.h"
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 
-#include <stdlib.h>
 #include <string.h>
-#include <time.h>
-
-#include <glib.h>
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-
 #include <loudmouth/loudmouth.h>
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/channel-factory-iface.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_IM
 
+#include "connection.h"
 #include "debug.h"
 #include "disco.h"
-#include "gabble-connection.h"
-#include "gabble-im-channel.h"
-#include <telepathy-glib/interfaces.h>
+#include "im-channel.h"
 #include "text-mixin.h"
-#include <telepathy-glib/channel-factory-iface.h>
 
 static void gabble_im_factory_iface_init (gpointer g_iface,
     gpointer iface_data);
@@ -56,7 +52,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef struct _GabbleImFactoryPrivate GabbleImFactoryPrivate;
 struct _GabbleImFactoryPrivate
 {
   GabbleConnection *conn;
@@ -70,21 +65,20 @@ struct _GabbleImFactoryPrivate
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_IM_FACTORY,\
                                 GabbleImFactoryPrivate))
 
-static GObject *gabble_im_factory_constructor (GType type, guint n_props,
-    GObjectConstructParam *props);
 
 static void
-gabble_im_factory_init (GabbleImFactory *fac)
+gabble_im_factory_init (GabbleImFactory *self)
 {
-  GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GABBLE_TYPE_IM_FACTORY,
+      GabbleImFactoryPrivate);
 
-  priv->channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+  self->priv->channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
                                           NULL, g_object_unref);
 
-  priv->message_cb = NULL;
+  self->priv->message_cb = NULL;
 
-  priv->conn = NULL;
-  priv->dispose_has_run = FALSE;
+  self->priv->conn = NULL;
+  self->priv->dispose_has_run = FALSE;
 }
 
 static GObject *
@@ -186,7 +180,8 @@ gabble_im_factory_class_init (GabbleImFactoryClass *gabble_im_factory_class)
 
 }
 
-static GabbleIMChannel *new_im_channel (GabbleImFactory *fac, TpHandle handle);
+static GabbleIMChannel *new_im_channel (GabbleImFactory *fac,
+    TpHandle handle, TpHandle initiator);
 
 static void im_channel_closed_cb (GabbleIMChannel *chan, gpointer user_data);
 
@@ -204,7 +199,7 @@ im_factory_message_cb (LmMessageHandler *handler,
 {
   GabbleImFactory *fac = GABBLE_IM_FACTORY (user_data);
   GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
   const gchar *from, *body;
@@ -252,7 +247,7 @@ im_factory_message_cb (LmMessageHandler *handler,
 
       DEBUG ("found no IM channel, creating one");
 
-      chan = new_im_channel (fac, handle);
+      chan = new_im_channel (fac, handle, handle);
     }
 
   g_assert (chan != NULL);
@@ -273,7 +268,7 @@ im_factory_message_cb (LmMessageHandler *handler,
       DEBUG ("got error sending to %s (handle %u), msgtype %u, body:\n%s",
          from, handle, msgtype, body);
 
-      tp_svc_channel_type_text_emit_send_error ((TpSvcChannelTypeText *)chan,
+      tp_svc_channel_type_text_emit_send_error ((TpSvcChannelTypeText *) chan,
           send_error, stamp, msgtype, body);
 
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
@@ -292,22 +287,40 @@ im_factory_message_cb (LmMessageHandler *handler,
  * im_channel_closed_cb:
  *
  * Signal callback for when an IM channel is closed. Removes the references
- * that #GabbleConnection holds to them.
+ * that #GabbleConnection holds to them - unless the channel has pending
+ * messages, in which case it is re-announced (so from the perspective of the
+ * D-Bus API, it was replaced by an identical channel).
  */
 static void
 im_channel_closed_cb (GabbleIMChannel *chan, gpointer user_data)
 {
-  GabbleImFactory *conn = GABBLE_IM_FACTORY (user_data);
-  GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (conn);
+  GabbleImFactory *self = GABBLE_IM_FACTORY (user_data);
+  GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (self);
   TpHandle contact_handle;
+  gboolean really_destroyed;
 
-  if (priv->channels)
+  DEBUG ("%p, channel %p", self, chan);
+
+  if (priv->channels != NULL)
     {
-      g_object_get (chan, "handle", &contact_handle, NULL);
+      g_object_get (chan,
+          "handle", &contact_handle,
+          "channel-destroyed", &really_destroyed,
+          NULL);
 
-      DEBUG ("removing channel with handle %d", contact_handle);
-
-      g_hash_table_remove (priv->channels, GINT_TO_POINTER (contact_handle));
+      if (really_destroyed)
+        {
+          DEBUG ("removing channel with handle %u", contact_handle);
+          g_hash_table_remove (priv->channels,
+              GUINT_TO_POINTER (contact_handle));
+        }
+      else
+        {
+          DEBUG ("reopening channel with handle %u due to pending messages",
+              contact_handle);
+          tp_channel_factory_iface_emit_new_channel (self,
+              (TpChannelIface *) chan, NULL);
+        }
     }
 }
 
@@ -315,17 +328,21 @@ im_channel_closed_cb (GabbleIMChannel *chan, gpointer user_data)
  * new_im_channel
  */
 static GabbleIMChannel *
-new_im_channel (GabbleImFactory *fac, TpHandle handle)
+new_im_channel (GabbleImFactory *fac,
+                TpHandle handle,
+                TpHandle initiator)
 {
   GabbleImFactoryPrivate *priv;
   TpBaseConnection *conn;
   GabbleIMChannel *chan;
   char *object_path;
 
-  g_assert (GABBLE_IS_IM_FACTORY (fac));
+  g_return_val_if_fail (GABBLE_IS_IM_FACTORY (fac), NULL);
+  g_return_val_if_fail (handle != 0, NULL);
+  g_return_val_if_fail (initiator != 0, NULL);
 
   priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
-  conn = (TpBaseConnection *)priv->conn;
+  conn = (TpBaseConnection *) priv->conn;
 
   object_path = g_strdup_printf ("%s/ImChannel%u",
       conn->object_path, handle);
@@ -334,6 +351,7 @@ new_im_channel (GabbleImFactory *fac, TpHandle handle)
                        "connection", priv->conn,
                        "object-path", object_path,
                        "handle", handle,
+                       "initiator-handle", initiator,
                        NULL);
 
   DEBUG ("object path %s", object_path);
@@ -342,7 +360,7 @@ new_im_channel (GabbleImFactory *fac, TpHandle handle)
 
   g_hash_table_insert (priv->channels, GINT_TO_POINTER (handle), chan);
 
-  tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *)chan,
+  tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *) chan,
       NULL);
 
   g_free (object_path);
@@ -447,8 +465,9 @@ gabble_im_factory_iface_request (TpChannelFactoryIface *iface,
 {
   GabbleImFactory *fac = GABBLE_IM_FACTORY (iface);
   GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
+  TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      base_conn, TP_HANDLE_TYPE_CONTACT);
   GabbleIMChannel *chan;
   TpChannelFactoryRequestStatus status;
 
@@ -467,7 +486,7 @@ gabble_im_factory_iface_request (TpChannelFactoryIface *iface,
   if (!chan)
     {
       status = TP_CHANNEL_FACTORY_REQUEST_STATUS_CREATED;
-      chan = new_im_channel (fac, handle);
+      chan = new_im_channel (fac, handle, base_conn->self_handle);
     }
 
   g_assert (chan);
