@@ -17,30 +17,27 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "media-factory.h"
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 
 #include <stdlib.h>
 #include <string.h>
-#include <time.h>
-
-#include <glib.h>
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-
 #include <loudmouth/loudmouth.h>
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/channel-factory-iface.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_MEDIA
 
+#include "connection.h"
 #include "debug.h"
-#include "gabble-connection.h"
-#include "gabble-media-channel.h"
+#include "media-channel.h"
 #include "namespaces.h"
-#include <telepathy-glib/interfaces.h>
 #include "text-mixin.h"
-#include <telepathy-glib/channel-factory-iface.h>
 #include "util.h"
 
 static void gabble_media_factory_iface_init (gpointer g_iface,
@@ -103,6 +100,9 @@ gabble_media_factory_init (GabbleMediaFactory *fac)
 }
 
 
+static void gabble_media_factory_close_all (GabbleMediaFactory *fac);
+
+
 static void
 gabble_media_factory_dispose (GObject *object)
 {
@@ -118,7 +118,7 @@ gabble_media_factory_dispose (GObject *object)
   g_assert (priv->jingle_cb == NULL);
   g_assert (priv->jingle_info_cb == NULL);
 
-  tp_channel_factory_iface_close_all (TP_CHANNEL_FACTORY_IFACE (object));
+  gabble_media_factory_close_all (fac);
   g_assert (priv->channels == NULL);
 
   if (priv->session_chans)
@@ -173,6 +173,10 @@ gabble_media_factory_set_property (GObject      *object,
   }
 }
 
+
+static void gabble_media_factory_constructed (GObject *object);
+
+
 static void
 gabble_media_factory_class_init (GabbleMediaFactoryClass *gabble_media_factory_class)
 {
@@ -182,6 +186,7 @@ gabble_media_factory_class_init (GabbleMediaFactoryClass *gabble_media_factory_c
   g_type_class_add_private (gabble_media_factory_class,
       sizeof (GabbleMediaFactoryPrivate));
 
+  object_class->constructed = gabble_media_factory_constructed;
   object_class->dispose = gabble_media_factory_dispose;
 
   object_class->get_property = gabble_media_factory_get_property;
@@ -202,7 +207,7 @@ gabble_media_factory_class_init (GabbleMediaFactoryClass *gabble_media_factory_c
 static gboolean _gabble_media_factory_sid_in_use (GabbleMediaFactory *fac,
     const gchar *sid);
 static GabbleMediaChannel *new_media_channel (GabbleMediaFactory *fac,
-    TpHandle handle);
+    TpHandle creator);
 static void media_channel_closed_cb (GabbleMediaChannel *chan,
     gpointer user_data);
 
@@ -222,7 +227,7 @@ media_factory_jingle_cb (LmMessageHandler *handler,
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (user_data);
   GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   LmMessageNode *iq_node, *session_node;
   const gchar *from, *id, *action, *sid, *resource;
   TpHandle handle = 0;
@@ -336,7 +341,7 @@ media_factory_jingle_cb (LmMessageHandler *handler,
     {
       if (chan_is_new)
         tp_channel_factory_iface_emit_new_channel (fac,
-            (TpChannelIface *)chan, NULL);
+            (TpChannelIface *) chan, NULL);
     }
   else
     {
@@ -474,7 +479,8 @@ media_channel_closed_cb (GabbleMediaChannel *chan, gpointer user_data)
  * Creates a new empty GabbleMediaChannel.
  */
 static GabbleMediaChannel *
-new_media_channel (GabbleMediaFactory *fac, TpHandle creator)
+new_media_channel (GabbleMediaFactory *fac,
+                   TpHandle creator)
 {
   GabbleMediaFactoryPrivate *priv;
   TpBaseConnection *conn;
@@ -482,9 +488,10 @@ new_media_channel (GabbleMediaFactory *fac, TpHandle creator)
   gchar *object_path;
 
   g_assert (GABBLE_IS_MEDIA_FACTORY (fac));
+  g_assert (creator != 0);
 
   priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
-  conn = (TpBaseConnection *)priv->conn;
+  conn = (TpBaseConnection *) priv->conn;
 
   object_path = g_strdup_printf ("%s/MediaChannel%u",
       conn->object_path, priv->channel_index);
@@ -663,9 +670,8 @@ jingle_info_iq_callback (LmMessageHandler *handler,
 
 
 static void
-gabble_media_factory_iface_close_all (TpChannelFactoryIface *iface)
+gabble_media_factory_close_all (GabbleMediaFactory *fac)
 {
-  GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (iface);
   GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
 
   DEBUG ("closing channels");
@@ -697,87 +703,111 @@ gabble_media_factory_iface_close_all (TpChannelFactoryIface *iface)
     }
 }
 
-static void
-gabble_media_factory_iface_connecting (TpChannelFactoryIface *iface)
-{
-  GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (iface);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
-
-  g_assert (priv->conn != NULL);
-  g_assert (priv->conn->lmconn != NULL);
-
-  DEBUG ("adding callbacks");
-
-  g_assert (priv->jingle_cb == NULL);
-  g_assert (priv->jingle_info_cb == NULL);
-
-  priv->jingle_cb = lm_message_handler_new (media_factory_jingle_cb, fac,
-      NULL);
-  lm_connection_register_message_handler (priv->conn->lmconn, priv->jingle_cb,
-                                          LM_MESSAGE_TYPE_IQ,
-                                          LM_HANDLER_PRIORITY_NORMAL);
-
-  priv->jingle_info_cb = lm_message_handler_new (jingle_info_iq_callback, fac,
-      NULL);
-  lm_connection_register_message_handler (priv->conn->lmconn,
-                                          priv->jingle_info_cb,
-                                          LM_MESSAGE_TYPE_IQ,
-                                          LM_HANDLER_PRIORITY_NORMAL);
-}
 
 static void
-gabble_media_factory_iface_connected (TpChannelFactoryIface *iface)
+connection_status_changed_cb (GabbleConnection *conn,
+                              guint status,
+                              guint reason,
+                              GabbleMediaFactory *self)
 {
-  GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (iface);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
-  gchar *stun_server = NULL;
-  guint stun_port = 0;
+  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (self);
 
-  g_object_get (priv->conn,
-      "stun-server", &stun_server,
-      "stun-port", &stun_port,
-      NULL);
+  switch (status)
+    {
+    case TP_CONNECTION_STATUS_CONNECTING:
+      g_assert (priv->conn != NULL);
+      g_assert (priv->conn->lmconn != NULL);
 
-  if (stun_server == NULL)
-    {
-      priv->get_stun_from_jingle = TRUE;
-    }
-  else
-    {
-      g_free (priv->stun_server);
-      priv->stun_server = stun_server;
-      priv->stun_port = stun_port;
-    }
+      DEBUG ("adding callbacks");
+      g_assert (priv->jingle_cb == NULL);
+      g_assert (priv->jingle_info_cb == NULL);
 
-  if (priv->conn->features & GABBLE_CONNECTION_FEATURES_GOOGLE_JINGLE_INFO)
-    {
-      jingle_info_send_request (fac);
+      priv->jingle_cb = lm_message_handler_new (media_factory_jingle_cb,
+          self, NULL);
+      lm_connection_register_message_handler (priv->conn->lmconn,
+          priv->jingle_cb, LM_MESSAGE_TYPE_IQ,
+          LM_HANDLER_PRIORITY_NORMAL);
+
+      priv->jingle_info_cb = lm_message_handler_new (
+          jingle_info_iq_callback, self, NULL);
+      lm_connection_register_message_handler (priv->conn->lmconn,
+          priv->jingle_info_cb, LM_MESSAGE_TYPE_IQ,
+          LM_HANDLER_PRIORITY_NORMAL);
+
+      break;
+
+    case TP_CONNECTION_STATUS_CONNECTED:
+        {
+          gchar *stun_server = NULL;
+          guint stun_port = 0;
+
+          g_object_get (priv->conn,
+              "stun-server", &stun_server,
+              "stun-port", &stun_port,
+              NULL);
+
+          if (stun_server == NULL)
+            {
+              priv->get_stun_from_jingle = TRUE;
+            }
+          else
+            {
+              g_free (priv->stun_server);
+              priv->stun_server = stun_server;
+              priv->stun_port = stun_port;
+            }
+
+          if (priv->conn->features &
+              GABBLE_CONNECTION_FEATURES_GOOGLE_JINGLE_INFO)
+            {
+              jingle_info_send_request (self);
+            }
+        }
+      break;
+
+    case TP_CONNECTION_STATUS_DISCONNECTED:
+      gabble_media_factory_close_all (self);
+
+      /* this can be called before we have ever been CONNECTING, so we need
+       * to guard it */
+      if (priv->jingle_cb != NULL)
+        {
+          DEBUG ("removing callbacks");
+          g_assert (priv->jingle_info_cb != NULL);
+
+          lm_connection_unregister_message_handler (priv->conn->lmconn,
+              priv->jingle_cb, LM_MESSAGE_TYPE_IQ);
+          lm_message_handler_unref (priv->jingle_cb);
+          priv->jingle_cb = NULL;
+
+          lm_connection_unregister_message_handler (priv->conn->lmconn,
+              priv->jingle_info_cb, LM_MESSAGE_TYPE_IQ);
+          lm_message_handler_unref (priv->jingle_info_cb);
+          priv->jingle_info_cb = NULL;
+        }
+      break;
     }
 }
 
+
 static void
-gabble_media_factory_iface_disconnected (TpChannelFactoryIface *iface)
+gabble_media_factory_constructed (GObject *object)
 {
-  GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (iface);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  void (*chain_up) (GObject *) =
+      G_OBJECT_CLASS (gabble_media_factory_parent_class)->constructed;
+  GabbleMediaFactory *self = GABBLE_MEDIA_FACTORY (object);
+  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (self);
 
-  DEBUG ("removing callbacks");
-
-  g_assert (priv->jingle_cb != NULL);
-  g_assert (priv->jingle_info_cb != NULL);
-
-  lm_connection_unregister_message_handler (priv->conn->lmconn,
-      priv->jingle_cb, LM_MESSAGE_TYPE_IQ);
-  lm_message_handler_unref (priv->jingle_cb);
-  priv->jingle_cb = NULL;
-
-  lm_connection_unregister_message_handler (priv->conn->lmconn,
-      priv->jingle_info_cb, LM_MESSAGE_TYPE_IQ);
-  lm_message_handler_unref (priv->jingle_info_cb);
-  priv->jingle_info_cb = NULL;
+  if (chain_up != NULL)
+    chain_up (object);
 
+  /* conn is guaranteed to live longer than the factory, so this
+   * never needs disconnecting */
+  g_signal_connect (priv->conn, "status-changed",
+      (GCallback) connection_status_changed_cb, object);
 }
 
+
 static void
 gabble_media_factory_iface_foreach (TpChannelFactoryIface *iface,
                                     TpChannelFunc foreach,
@@ -805,7 +835,7 @@ gabble_media_factory_iface_request (TpChannelFactoryIface *iface,
 {
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (iface);
   GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   GabbleMediaChannel *chan = NULL;
 
   if (strcmp (chan_type, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA))
@@ -820,7 +850,7 @@ gabble_media_factory_iface_request (TpChannelFactoryIface *iface,
     {
       chan = new_media_channel (fac, conn->self_handle);
 
-      if (!_gabble_media_channel_add_member ((GObject *)chan, handle, "",
+      if (!_gabble_media_channel_add_member ((GObject *) chan, handle, "",
             error))
         {
           gabble_media_channel_close (chan);
@@ -834,7 +864,7 @@ gabble_media_factory_iface_request (TpChannelFactoryIface *iface,
     }
 
   g_assert (chan != NULL);
-  tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *)chan,
+  tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *) chan,
       request);
 
   *ret = TP_CHANNEL_IFACE (chan);
@@ -847,10 +877,8 @@ gabble_media_factory_iface_init (gpointer g_iface,
 {
   TpChannelFactoryIfaceClass *klass = (TpChannelFactoryIfaceClass *) g_iface;
 
-  klass->close_all = gabble_media_factory_iface_close_all;
-  klass->connected = gabble_media_factory_iface_connected;
-  klass->connecting = gabble_media_factory_iface_connecting;
-  klass->disconnected = gabble_media_factory_iface_disconnected;
+  klass->close_all =
+      (TpChannelFactoryIfaceProc) gabble_media_factory_close_all;
   klass->foreach = gabble_media_factory_iface_foreach;
   klass->request = gabble_media_factory_iface_request;
 }
