@@ -17,15 +17,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "muc-factory.h"
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 
-#include <stdlib.h>
 #include <string.h>
-#include <time.h>
 
-#include <glib.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 #include <loudmouth/loudmouth.h>
@@ -33,15 +31,15 @@
 #include <telepathy-glib/interfaces.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_MUC
+
+#include "connection.h"
 #include "conn-olpc.h"
 #include "debug.h"
 #include "disco.h"
-#include "extensions/extensions.h"
-#include "gabble-connection.h"
-#include "gabble-muc-channel.h"
-#include "gabble-roomlist-channel.h"
+#include "muc-channel.h"
 #include "namespaces.h"
 #include "presence-cache.h"
+#include "roomlist-channel.h"
 #include "text-mixin.h"
 #include "tubes-channel.h"
 #include "util.h"
@@ -76,6 +74,7 @@ struct _GabbleMucFactoryPrivate
    * text channel is created.
    * GabbleMucChannel * => GabbleTubesChannel * */
   GHashTable *text_needed_for_tubes;
+  /* GabbleDiscoRequest * => NULL (used as a set) */
   GHashTable *disco_requests;
 
   gboolean dispose_has_run;
@@ -110,19 +109,6 @@ gabble_muc_factory_init (GabbleMucFactory *fac)
   priv->dispose_has_run = FALSE;
 }
 
-static GObject *
-gabble_muc_factory_constructor (GType type, guint n_props,
-                                GObjectConstructParam *props)
-{
-  GObject *obj;
-  /* GabbleMucFactoryPrivate *priv; */
-
-  obj = G_OBJECT_CLASS (gabble_muc_factory_parent_class)->
-           constructor (type, n_props, props);
-  /* priv = GABBLE_MUC_FACTORY_GET_PRIVATE (obj); */
-
-  return obj;
-}
 
 static void
 cancel_disco_request (gpointer key, gpointer value, gpointer user_data)
@@ -133,6 +119,10 @@ cancel_disco_request (gpointer key, gpointer value, gpointer user_data)
   gabble_disco_cancel_request (disco, request);
 }
 
+
+static void gabble_muc_factory_close_all (GabbleMucFactory *fac);
+
+
 static void
 gabble_muc_factory_dispose (GObject *object)
 {
@@ -145,7 +135,7 @@ gabble_muc_factory_dispose (GObject *object)
   DEBUG ("dispose called");
   priv->dispose_has_run = TRUE;
 
-  tp_channel_factory_iface_close_all (TP_CHANNEL_FACTORY_IFACE (object));
+  gabble_muc_factory_close_all (fac);
   g_assert (priv->text_channels == NULL);
   g_assert (priv->tubes_channels == NULL);
   g_assert (priv->text_needed_for_tubes == NULL);
@@ -329,7 +319,11 @@ muc_join_error_cb (GabbleMucChannel *chan,
  * new_muc_channel
  */
 static GabbleMucChannel *
-new_muc_channel (GabbleMucFactory *fac, TpHandle handle, gboolean invite_self)
+new_muc_channel (GabbleMucFactory *fac,
+                 TpHandle handle,
+                 gboolean invited,
+                 TpHandle inviter,
+                 const gchar *message)
 {
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
   TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
@@ -345,10 +339,12 @@ new_muc_channel (GabbleMucFactory *fac, TpHandle handle, gboolean invite_self)
   DEBUG ("creating new chan, object path %s", object_path);
 
   chan = g_object_new (GABBLE_TYPE_MUC_CHANNEL,
-      "connection", priv->conn,
+       "connection", priv->conn,
        "object-path", object_path,
        "handle", handle,
-       "invite-self", invite_self,
+       "invited", invited,
+       "initiator-handle", invited ? inviter : conn->self_handle,
+       "invitation-message", message,
        NULL);
 
   g_signal_connect (chan, "closed", (GCallback) muc_channel_closed_cb, fac);
@@ -357,7 +353,11 @@ new_muc_channel (GabbleMucFactory *fac, TpHandle handle, gboolean invite_self)
 
   g_free (object_path);
 
-  g_signal_connect (chan, "ready", G_CALLBACK (muc_ready_cb), fac);
+  if (_gabble_muc_channel_is_ready (chan))
+    muc_ready_cb (chan, fac);
+  else
+    g_signal_connect (chan, "ready", G_CALLBACK (muc_ready_cb), fac);
+
   g_signal_connect (chan, "join-error", G_CALLBACK (muc_join_error_cb),
                     fac);
 
@@ -372,7 +372,8 @@ new_muc_channel (GabbleMucFactory *fac, TpHandle handle, gboolean invite_self)
 static GabbleTubesChannel *
 new_tubes_channel (GabbleMucFactory *fac,
                    TpHandle room,
-                   GabbleMucChannel *muc)
+                   GabbleMucChannel *muc,
+                   TpHandle initiator)
 {
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
   TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
@@ -393,6 +394,7 @@ new_tubes_channel (GabbleMucFactory *fac,
       "handle", room,
       "handle-type", TP_HANDLE_TYPE_ROOM,
       "muc", muc,
+      "initiator-handle", initiator,
       NULL);
 
   g_signal_connect (chan, "closed", (GCallback) tubes_channel_closed_cb, fac);
@@ -427,13 +429,14 @@ do_invite (GabbleMucFactory *fac,
   if (g_hash_table_lookup (priv->text_channels,
         GUINT_TO_POINTER (room_handle)) == NULL)
     {
-      GabbleMucChannel *chan = new_muc_channel (fac, room_handle, FALSE);
-      _gabble_muc_channel_handle_invited (chan, inviter_handle, reason);
+      new_muc_channel (fac, room_handle, TRUE, inviter_handle, reason);
     }
   else
     {
       DEBUG ("ignoring invite to room \"%s\"; we're already there", room);
     }
+
+  tp_handle_unref (room_repo, room_handle);
 }
 
 struct DiscoInviteData {
@@ -561,9 +564,6 @@ process_muc_invite (GabbleMucFactory *fac,
   if (reason_node != NULL)
     reason = lm_message_node_get_value (reason_node);
 
-  if (reason == NULL)
-    reason = "";
-
   /* create the channel */
   room = gabble_remove_resource (from);
   do_invite (fac, room, inviter_handle, reason);
@@ -895,7 +895,10 @@ muc_factory_presence_cb (LmMessageHandler *handler,
              * to create a tubes channel */
             return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 
-          tubes_chan = new_tubes_channel (fac, room_handle, muc_chan);
+          /* MUC Tubes channels (as opposed to the individual tubes) don't
+           * have a well-defined initiator (they're a consensus) so use 0 */
+          tubes_chan = new_tubes_channel (fac, room_handle, muc_chan,
+              0);
           tp_channel_factory_iface_emit_new_channel (fac,
               (TpChannelIface *) tubes_chan, NULL);
         }
@@ -966,9 +969,8 @@ make_roomlist_channel (GabbleMucFactory *fac, gpointer request)
 
 
 static void
-gabble_muc_factory_iface_close_all (TpChannelFactoryIface *iface)
+gabble_muc_factory_close_all (GabbleMucFactory *fac)
 {
-  GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
 
   DEBUG ("closing channels");
@@ -1001,57 +1003,78 @@ gabble_muc_factory_iface_close_all (TpChannelFactoryIface *iface)
     }
 }
 
+
 static void
-gabble_muc_factory_iface_connecting (TpChannelFactoryIface *iface)
+connection_status_changed_cb (GabbleConnection *conn,
+                              guint status,
+                              guint reason,
+                              GabbleMucFactory *self)
 {
-  GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
-  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (self);
 
-  DEBUG ("adding callbacks");
+  switch (status)
+    {
+    case TP_CONNECTION_STATUS_CONNECTING:
+      DEBUG ("adding callbacks");
+      g_assert (priv->message_cb == NULL);
+      g_assert (priv->presence_cb == NULL);
 
-  g_assert (priv->message_cb == NULL);
-  g_assert (priv->presence_cb == NULL);
+      priv->message_cb = lm_message_handler_new (muc_factory_message_cb,
+          self, NULL);
+      lm_connection_register_message_handler (priv->conn->lmconn,
+          priv->message_cb, LM_MESSAGE_TYPE_MESSAGE,
+          LM_HANDLER_PRIORITY_NORMAL);
 
-  priv->message_cb = lm_message_handler_new (muc_factory_message_cb, fac,
-      NULL);
-  lm_connection_register_message_handler (priv->conn->lmconn, priv->message_cb,
-                                          LM_MESSAGE_TYPE_MESSAGE,
-                                          LM_HANDLER_PRIORITY_NORMAL);
+      priv->presence_cb = lm_message_handler_new (
+          muc_factory_presence_cb, self, NULL);
+      lm_connection_register_message_handler (priv->conn->lmconn,
+          priv->presence_cb, LM_MESSAGE_TYPE_PRESENCE,
+          LM_HANDLER_PRIORITY_NORMAL);
 
-  priv->presence_cb = lm_message_handler_new (muc_factory_presence_cb,
-      fac, NULL);
-  lm_connection_register_message_handler (priv->conn->lmconn,
-      priv->presence_cb, LM_MESSAGE_TYPE_PRESENCE, LM_HANDLER_PRIORITY_NORMAL);
+      break;
+
+    case TP_CONNECTION_STATUS_DISCONNECTED:
+      gabble_muc_factory_close_all (self);
+
+      /* this can be called before we have ever been CONNECTING, so we need
+       * to guard it */
+      if (priv->message_cb != NULL)
+        {
+          DEBUG ("removing callbacks");
+          g_assert (priv->presence_cb != NULL);
+
+          lm_connection_unregister_message_handler (priv->conn->lmconn,
+              priv->message_cb, LM_MESSAGE_TYPE_MESSAGE);
+          lm_message_handler_unref (priv->message_cb);
+          priv->message_cb = NULL;
+
+          lm_connection_unregister_message_handler (priv->conn->lmconn,
+              priv->presence_cb, LM_MESSAGE_TYPE_PRESENCE);
+          lm_message_handler_unref (priv->presence_cb);
+          priv->presence_cb = NULL;
+        }
+      break;
+    }
 }
 
 
-static void
-gabble_muc_factory_iface_connected (TpChannelFactoryIface *iface)
+static GObject *
+gabble_muc_factory_constructor (GType type, guint n_props,
+                                GObjectConstructParam *props)
 {
-  /* nothing to do */
+  GObject *obj = G_OBJECT_CLASS (gabble_muc_factory_parent_class)->
+           constructor (type, n_props, props);
+  GabbleMucFactory *self = GABBLE_MUC_FACTORY (obj);
+  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (self);
+
+  /* conn is guaranteed to live longer than the factory, so this
+   * never needs disconnecting */
+  g_signal_connect (priv->conn, "status-changed",
+      (GCallback) connection_status_changed_cb, obj);
+
+  return obj;
 }
 
-static void
-gabble_muc_factory_iface_disconnected (TpChannelFactoryIface *iface)
-{
-  GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
-  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
-
-  DEBUG ("removing callbacks");
-
-  g_assert (priv->message_cb != NULL);
-  g_assert (priv->presence_cb != NULL);
-
-  lm_connection_unregister_message_handler (priv->conn->lmconn,
-      priv->message_cb, LM_MESSAGE_TYPE_MESSAGE);
-  lm_message_handler_unref (priv->message_cb);
-  priv->message_cb = NULL;
-
-  lm_connection_unregister_message_handler (priv->conn->lmconn,
-      priv->presence_cb, LM_MESSAGE_TYPE_PRESENCE);
-  lm_message_handler_unref (priv->presence_cb);
-  priv->presence_cb = NULL;
-}
 
 struct _ForeachData
 {
@@ -1090,7 +1113,9 @@ gabble_muc_factory_iface_foreach (TpChannelFactoryIface *iface,
 /**
  * ensure_muc_channel:
  *
- * Create a MUC channel. Return TRUE if it already existed, or return FALSE
+ * Create a MUC channel in response to RequestChannel.
+ *
+ * Return TRUE if it already existed, or return FALSE
  * if it needed to be created (so isn't ready yet).
  */
 static gboolean
@@ -1099,12 +1124,16 @@ ensure_muc_channel (GabbleMucFactory *fac,
                     TpHandle handle,
                     GabbleMucChannel **ret)
 {
+  TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
+
   *ret = g_hash_table_lookup (priv->text_channels, GINT_TO_POINTER (handle));
-  if (!*ret)
+
+  if (*ret == NULL)
     {
-      *ret = new_muc_channel (fac, handle, TRUE);
+      *ret = new_muc_channel (fac, handle, FALSE, base_conn->self_handle, NULL);
       return FALSE;
     }
+
   if (_gabble_muc_channel_is_ready (*ret))
     return TRUE;
   else
@@ -1122,8 +1151,9 @@ gabble_muc_factory_iface_request (TpChannelFactoryIface *iface,
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
-  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_ROOM);
+  TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
+  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (base_conn,
+      TP_HANDLE_TYPE_ROOM);
   GabbleMucChannel *text_chan;
   GabbleTubesChannel *tubes_chan;
 
@@ -1176,14 +1206,16 @@ gabble_muc_factory_iface_request (TpChannelFactoryIface *iface,
         {
           if (ensure_muc_channel (fac, priv, handle, &text_chan))
             {
-              tubes_chan = new_tubes_channel (fac, handle, text_chan);
+              tubes_chan = new_tubes_channel (fac, handle, text_chan,
+                  base_conn->self_handle);
               *ret = TP_CHANNEL_IFACE (tubes_chan);
               tp_channel_factory_iface_emit_new_channel (fac, *ret, NULL);
               return TP_CHANNEL_FACTORY_REQUEST_STATUS_CREATED;
             }
           else
             {
-              tubes_chan = new_tubes_channel (fac, handle, text_chan);
+              tubes_chan = new_tubes_channel (fac, handle, text_chan,
+                  base_conn->self_handle);
               g_hash_table_insert (priv->text_needed_for_tubes,
                   text_chan, tubes_chan);
               return TP_CHANNEL_FACTORY_REQUEST_STATUS_QUEUED;
@@ -1205,7 +1237,7 @@ gabble_muc_factory_handle_si_stream_request (GabbleMucFactory *self,
 {
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (self);
   TpHandleRepoIface *room_repo = tp_base_connection_get_handles (
-     (TpBaseConnection*) priv->conn, TP_HANDLE_TYPE_ROOM);
+     (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_ROOM);
   GabbleTubesChannel *chan;
 
   g_return_if_fail (tp_handle_is_valid (room_repo, room_handle, NULL));
@@ -1240,10 +1272,8 @@ gabble_muc_factory_iface_init (gpointer g_iface,
 {
   TpChannelFactoryIfaceClass *klass = (TpChannelFactoryIfaceClass *) g_iface;
 
-  klass->close_all = gabble_muc_factory_iface_close_all;
-  klass->connecting = gabble_muc_factory_iface_connecting;
-  klass->connected = gabble_muc_factory_iface_connected;
-  klass->disconnected = gabble_muc_factory_iface_disconnected;
+  klass->close_all =
+      (TpChannelFactoryIfaceProc) gabble_muc_factory_close_all;
   klass->foreach = gabble_muc_factory_iface_foreach;
   klass->request = gabble_muc_factory_iface_request;
 }
