@@ -18,24 +18,37 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "presence-cache.h"
 
 #include <stdlib.h>
 #include <string.h>
 
-/* when five DIFFERENT guys report the same caps for a given bundle, it'll
-be enough */
+/* When five DIFFERENT guys report the same caps for a given bundle, it'll
+ * be enough. But if only ONE guy use the verification string (XEP-0115 v1.5),
+ * it'll be enough too.
+ */
 #define CAPABILITY_BUNDLE_ENOUGH_TRUST 5
 #define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
 
+#include <dbus/dbus-glib.h>
 #include <telepathy-glib/intset.h>
 
+#define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
+
+#include "caps-hash.h"
 #include "debug.h"
-#include "disco.h" /* \o\ \o/ /o/ */
+#include "disco.h"
+#include "gabble-signals-marshal.h"
 #include "namespaces.h"
 #include "util.h"
+#include "roster.h"
 
-#include "gabble-signals-marshal.h"
+/* When five DIFFERENT guys report the same caps for a given bundle, it'll
+ * be enough. But if only ONE guy use the verification string (XEP-0115 v1.5),
+ * it'll be enough too.
+ */
+#define CAPABILITY_BUNDLE_ENOUGH_TRUST 5
 
 G_DEFINE_TYPE (GabblePresenceCache, gabble_presence_cache, G_TYPE_OBJECT);
 
@@ -49,7 +62,7 @@ enum
 /* signal enum */
 enum
 {
-  PRESENCE_UPDATE,
+  PRESENCES_UPDATED,
   NICKNAME_UPDATE,
   CAPABILITIES_UPDATE,
   AVATAR_UPDATE,
@@ -58,10 +71,7 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-#define GABBLE_PRESENCE_CACHE_PRIV(account) \
-  ((GabblePresenceCachePrivate *)account->priv)
-
-typedef struct _GabblePresenceCachePrivate GabblePresenceCachePrivate;
+#define GABBLE_PRESENCE_CACHE_PRIV(account) ((account)->priv)
 
 struct _GabblePresenceCachePrivate
 {
@@ -89,6 +99,8 @@ struct _DiscoWaiter
   gchar *resource;
   guint serial;
   gboolean disco_requested;
+  gchar *hash;
+  gchar *ver;
 };
 
 /**
@@ -98,6 +110,8 @@ static DiscoWaiter *
 disco_waiter_new (TpHandleRepoIface *repo,
                   TpHandle handle,
                   const gchar *resource,
+                  const gchar *hash,
+                  const gchar *ver,
                   guint serial)
 {
   DiscoWaiter *waiter;
@@ -109,6 +123,8 @@ disco_waiter_new (TpHandleRepoIface *repo,
   waiter->repo = repo;
   waiter->handle = handle;
   waiter->resource = g_strdup (resource);
+  waiter->hash = g_strdup (hash);
+  waiter->ver = g_strdup (ver);
   waiter->serial = serial;
 
   DEBUG ("created waiter %p for handle %u with serial %u", waiter, handle,
@@ -128,6 +144,8 @@ disco_waiter_free (DiscoWaiter *waiter)
   tp_handle_unref (waiter->repo, waiter->handle);
 
   g_free (waiter->resource);
+  g_free (waiter->hash);
+  g_free (waiter->ver);
   g_slice_free (DiscoWaiter, waiter);
 }
 
@@ -155,7 +173,16 @@ disco_waiter_list_get_request_count (GSList *list)
       DiscoWaiter *waiter = (DiscoWaiter *) i->data;
 
       if (waiter->disco_requested)
-        c++;
+        {
+          if (!tp_strdiff (waiter->hash, "sha-1"))
+            /* One waiter is enough if
+             * 1. the request has a verification string
+             * 2. the hash algorithm is supported
+             */
+            c += CAPABILITY_BUNDLE_ENOUGH_TRUST;
+          else
+            c++;
+        }
     }
 
   return c;
@@ -197,7 +224,8 @@ capability_info_free (CapabilityInfo *info)
 
 static guint
 capability_info_recvd (GabblePresenceCache *cache, const gchar *node,
-        TpHandle handle, GabblePresenceCapabilities caps)
+        TpHandle handle, GabblePresenceCapabilities caps,
+        guint trust_inc)
 {
   CapabilityInfo *info = capability_info_get (cache, node, caps);
 
@@ -212,7 +240,7 @@ capability_info_recvd (GabblePresenceCache *cache, const gchar *node,
   if (!tp_intset_is_member (info->guys, handle))
     {
       tp_intset_add (info->guys, handle);
-      info->trust++;
+      info->trust += trust_inc;
     }
 
   return info->trust;
@@ -263,13 +291,13 @@ gabble_presence_cache_class_init (GabblePresenceCacheClass *klass)
                                    PROP_CONNECTION,
                                    param_spec);
 
-  signals[PRESENCE_UPDATE] = g_signal_new (
-    "presence-update",
+  signals[PRESENCES_UPDATED] = g_signal_new (
+    "presences-updated",
     G_TYPE_FROM_CLASS (klass),
     G_SIGNAL_RUN_LAST,
     0,
     NULL, NULL,
-    g_cclosure_marshal_VOID__UINT, G_TYPE_NONE, 1, G_TYPE_UINT);
+    g_cclosure_marshal_VOID__BOXED, G_TYPE_NONE, 1, DBUS_TYPE_G_UINT_ARRAY);
   signals[NICKNAME_UPDATE] = g_signal_new (
     "nickname-update",
     G_TYPE_FROM_CLASS (klass),
@@ -402,7 +430,7 @@ gabble_presence_cache_set_property (GObject     *object,
     case PROP_CONNECTION:
       priv->conn = g_value_get_object (value);
       contact_repo = tp_base_connection_get_handles (
-          (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+          (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
       new_presence_handles = tp_handle_set_new (contact_repo);
 
@@ -612,26 +640,39 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
 }
 
 static GSList *
-_extract_cap_bundles (LmMessageNode *lm_node)
+_parse_cap_bundles (
+    LmMessageNode *lm_node,
+    const gchar **hash,
+    const gchar **ver)
 {
-  const gchar *node, *ver, *ext;
+  const gchar *node, *ext;
   GSList *uris = NULL;
   LmMessageNode *cap_node;
 
+  *hash = NULL;
+  *ver = NULL;
+
   cap_node = lm_message_node_get_child_with_namespace (lm_node, "c", NS_CAPS);
 
   if (NULL == cap_node)
-    return NULL;
+      return NULL;
+
+  *hash = lm_message_node_get_attribute (cap_node, "hash");
 
   node = lm_message_node_get_attribute (cap_node, "node");
 
   if (NULL == node)
     return NULL;
 
-  ver = lm_message_node_get_attribute (cap_node, "ver");
+  *ver = lm_message_node_get_attribute (cap_node, "ver");
 
-  if (NULL != ver)
-    uris = g_slist_prepend (uris, g_strdup_printf ("%s#%s", node, ver));
+  if (NULL != *ver)
+    uris = g_slist_prepend (uris, g_strdup_printf ("%s#%s", node, *ver));
+
+  /* If there is a hash, the remote contact uses XEP-0115 v1.5 and the 'ext'
+   * attribute MUST be ignored. */
+  if (NULL != *hash)
+    return uris;
 
   ext = lm_message_node_get_attribute (cap_node, "ext");
 
@@ -660,19 +701,21 @@ _caps_disco_cb (GabbleDisco *disco,
                 gpointer user_data)
 {
   GSList *waiters, *i;
+  DiscoWaiter *waiter_self;
   LmMessageNode *child;
   GabblePresenceCache *cache;
   GabblePresenceCachePrivate *priv;
   TpHandleRepoIface *contact_repo;
   gchar *full_jid = NULL;
   GabblePresenceCapabilities caps = 0;
-  guint trust;
+  guint trust, trust_inc;
   TpHandle handle = 0;
+  gboolean bad_hash = FALSE;
 
   cache = GABBLE_PRESENCE_CACHE (user_data);
   priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   if (NULL == node)
     {
@@ -713,9 +756,11 @@ _caps_disco_cb (GabbleDisco *disco,
         }
       else
         {
+          /* The contact sends us an error and we don't have any other
+           * contacts to send the discovery request on the same node. We
+           * cannot get the caps for this node. */
           DEBUG ("failed to find a suitable candidate to retry disco "
               "request for URI %s", node);
-          /* FIXME do something very clever here? */
           g_hash_table_remove (priv->disco_pending, node);
         }
 
@@ -769,7 +814,54 @@ _caps_disco_cb (GabbleDisco *disco,
       goto OUT;
     }
 
-  trust = capability_info_recvd (cache, node, handle, caps);
+  waiter_self = NULL;
+  for (i = waiters; NULL != i;  i = i->next)
+    {
+      DiscoWaiter *waiter;
+
+      waiter = (DiscoWaiter *) i->data;
+      if (waiter->handle == handle)
+        {
+          waiter_self = waiter;
+          break;
+        }
+    }
+  if (NULL == waiter_self)
+    {
+      DEBUG ("Ignoring non requested disco reply");
+      goto OUT;
+    }
+
+  /* Only 'sha-1' is mandatory to implement by XEP-0115. If the remote contact
+   * uses another hash algorithm, don't check the hash and fallback to the old
+   * method. The hash method is not included in the discovery request nor
+   * response but we saved it in disco_pending when we received the presence
+   * stanza. */
+  if (!tp_strdiff (waiter_self->hash, "sha-1"))
+    {
+      const gchar *computed_hash;
+      trust_inc = CAPABILITY_BUNDLE_ENOUGH_TRUST;
+
+      computed_hash = caps_hash_compute_from_lm_node (query_result);
+
+      if (g_str_equal (waiter_self->ver, computed_hash))
+        {
+          trust = capability_info_recvd (cache, node, handle, caps, trust_inc);
+        }
+      else
+        {
+          /* The received reply does not match the */
+          g_warning ("The announced verification string '%s' does not match "
+              "our hash '%s'.", waiter_self->ver, computed_hash);
+          trust = 0;
+          bad_hash = TRUE;
+        }
+    }
+  else
+    {
+      trust_inc = 1;
+      trust = capability_info_recvd (cache, node, handle, caps, trust_inc);
+    }
 
   for (i = waiters; NULL != i;)
     {
@@ -784,19 +876,25 @@ _caps_disco_cb (GabbleDisco *disco,
           gpointer key;
           gpointer value;
 
-          /* trusted reply */
-          presence = gabble_presence_cache_get (cache, waiter->handle);
+          if (!bad_hash)
+            {
+              /* trusted reply */
+              presence = gabble_presence_cache_get (cache, waiter->handle);
 
-          if (presence)
-          {
-            GabblePresenceCapabilities save_caps = presence->caps;
-            DEBUG ("setting caps for %d (%s) to %d", handle, jid, caps);
-            gabble_presence_set_capabilities (presence, waiter->resource,caps,
-              waiter->serial);
-            DEBUG ("caps for %d (%s) now %d", handle, jid, presence->caps);
-            g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
-              waiter->handle, save_caps, presence->caps);
-          }
+              if (presence)
+              {
+                GabblePresenceCapabilities save_caps = presence->caps;
+                DEBUG ("setting caps for %d (thanks to %d %s) to "
+                    "%d (save_caps %d)",
+                    waiter->handle, handle, jid, caps, save_caps);
+                gabble_presence_set_capabilities (presence,
+                    waiter->resource,caps, waiter->serial);
+                DEBUG ("caps for %d (thanks to %d %s) now %d", waiter->handle,
+                    handle, jid, presence->caps);
+                g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
+                  waiter->handle, save_caps, presence->caps);
+              }
+            }
 
           tmp = i;
           i = i->next;
@@ -812,7 +910,7 @@ _caps_disco_cb (GabbleDisco *disco,
 
           disco_waiter_free (waiter);
         }
-      else if (trust + disco_waiter_list_get_request_count (waiters) - 1
+      else if (trust + disco_waiter_list_get_request_count (waiters) - trust_inc
           < CAPABILITY_BUNDLE_ENOUGH_TRUST)
         {
           /* if the possible trust, not counting this guy, is too low,
@@ -857,6 +955,8 @@ static void
 _process_caps_uri (GabblePresenceCache *cache,
                    const gchar *from,
                    const gchar *uri,
+                   const gchar *hash,
+                   const gchar *ver,
                    TpHandle handle,
                    const gchar *resource,
                    guint serial)
@@ -867,7 +967,7 @@ _process_caps_uri (GabblePresenceCache *cache,
 
   priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   info = capability_info_get (cache, uri, 0);
 
   if (info->trust >= CAPABILITY_BUNDLE_ENOUGH_TRUST
@@ -921,7 +1021,8 @@ _process_caps_uri (GabblePresenceCache *cache,
         }
 
       waiters = (GSList *) value;
-      waiter = disco_waiter_new (contact_repo, handle, resource, serial);
+      waiter = disco_waiter_new (contact_repo, handle, resource,
+          hash, ver, serial);
       waiters = g_slist_prepend (waiters, waiter);
       g_hash_table_insert (priv->disco_pending, key, waiters);
 
@@ -954,6 +1055,7 @@ _process_caps (GabblePresenceCache *cache,
   GabblePresenceCachePrivate *priv;
   GabblePresenceCapabilities old_caps = 0;
   guint serial;
+  const gchar *hash, *ver;
 
   priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   serial = priv->caps_serial++;
@@ -962,14 +1064,15 @@ _process_caps (GabblePresenceCache *cache,
   if (resource != NULL)
     resource++;
 
-  uris = _extract_cap_bundles (lm_node);
+  uris = _parse_cap_bundles (lm_node, &hash, &ver);
 
   if (presence)
       old_caps = presence->caps;
 
   for (i = uris; NULL != i; i = i->next)
     {
-      _process_caps_uri (cache, from, (gchar *) i->data, handle, resource, serial);
+      _process_caps_uri (cache, from, (gchar *) i->data, hash, ver, handle,
+          resource, serial);
       g_free (i->data);
 
     }
@@ -1001,6 +1104,7 @@ _parse_presence_message (GabblePresenceCache *cache,
                          const gchar *from,
                          LmMessage *message)
 {
+  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   gint8 priority = 0;
   const gchar *resource, *status_message = NULL;
   LmMessageNode *presence_node, *child_node;
@@ -1056,12 +1160,22 @@ _parse_presence_message (GabblePresenceCache *cache,
       break;
 
     case LM_MESSAGE_SUB_TYPE_ERROR:
-      NODE_DEBUG (presence_node, "setting contact offline due to error");
-      /* fall through */
+      NODE_DEBUG (presence_node, "Received error presence");
+      gabble_presence_cache_update (cache, handle, resource,
+          GABBLE_PRESENCE_ERROR, status_message, priority);
+
+      ret = LM_HANDLER_RESULT_REMOVE_MESSAGE;
+      break;
 
     case LM_MESSAGE_SUB_TYPE_UNAVAILABLE:
+      if (gabble_roster_handle_get_subscription (priv->conn->roster, handle)
+        & GABBLE_ROSTER_SUBSCRIPTION_FROM)
+        presence_id = GABBLE_PRESENCE_OFFLINE;
+      else
+        presence_id = GABBLE_PRESENCE_UNKNOWN;
+
       gabble_presence_cache_update (cache, handle, resource,
-          GABBLE_PRESENCE_OFFLINE, status_message, priority);
+          presence_id, status_message, priority);
 
       ret = LM_HANDLER_RESULT_REMOVE_MESSAGE;
       break;
@@ -1127,7 +1241,7 @@ gabble_presence_cache_lm_message_cb (LmMessageHandler *handler,
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (user_data);
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   const char *from;
   LmHandlerResult ret;
   TpHandle handle;
@@ -1181,7 +1295,7 @@ gabble_presence_cache_get (GabblePresenceCache *cache, TpHandle handle)
 {
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   g_assert (tp_handle_is_valid (contact_repo, handle, NULL));
 
@@ -1195,7 +1309,7 @@ gabble_presence_cache_maybe_remove (
 {
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabblePresence *presence;
 
   presence = gabble_presence_cache_get (cache, handle);
@@ -1230,8 +1344,8 @@ _cache_insert (
   return presence;
 }
 
-void
-gabble_presence_cache_update (
+static gboolean
+gabble_presence_cache_do_update (
     GabblePresenceCache *cache,
     TpHandle handle,
     const gchar *resource,
@@ -1241,10 +1355,11 @@ gabble_presence_cache_update (
 {
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   const gchar *jid;
   GabblePresence *presence;
   GabblePresenceCapabilities caps_before;
+  gboolean ret = FALSE;
 
   jid = tp_handle_inspect (contact_repo, handle);
   DEBUG ("%s (%d) resource %s prio %d presence %d message \"%s\"",
@@ -1257,17 +1372,83 @@ gabble_presence_cache_update (
 
   caps_before = presence->caps;
 
-  if (gabble_presence_update (presence, resource, presence_id, status_message,
-        priority))
-    g_signal_emit (cache, signals[PRESENCE_UPDATE], 0, handle);
+  ret = gabble_presence_update (presence, resource, presence_id,
+      status_message, priority);
 
   if (caps_before != presence->caps)
     g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0, handle,
         caps_before, presence->caps);
 
+  return ret;
+}
+
+void
+gabble_presence_cache_update (
+    GabblePresenceCache *cache,
+    TpHandle handle,
+    const gchar *resource,
+    GabblePresenceId presence_id,
+    const gchar *status_message,
+    gint8 priority)
+{
+  if (gabble_presence_cache_do_update (cache, handle, resource, presence_id,
+      status_message, priority))
+    {
+      GArray *handles;
+
+      handles = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
+
+      g_array_append_val (handles, handle);
+      g_signal_emit (cache, signals[PRESENCES_UPDATED], 0, handles);
+      g_array_free (handles, TRUE);
+    }
+
   gabble_presence_cache_maybe_remove (cache, handle);
 }
 
+void
+gabble_presence_cache_update_many (
+    GabblePresenceCache *cache,
+    const GArray *contact_handles,
+    const gchar *resource,
+    GabblePresenceId presence_id,
+    const gchar *status_message,
+    gint8 priority)
+{
+  GArray *updated;
+  guint i;
+
+  updated = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle),
+      contact_handles->len);
+
+  for (i = 0 ; i < contact_handles->len ; i++)
+    {
+      TpHandle handle;
+
+      handle = g_array_index (contact_handles, TpHandle, i);
+
+      if (gabble_presence_cache_do_update (cache, handle, resource,
+          presence_id, status_message, priority))
+        {
+          g_array_append_val (updated, handle);
+        }
+    }
+
+  if (updated->len > 0)
+    g_signal_emit (cache, signals[PRESENCES_UPDATED], 0, updated);
+
+  g_array_free (updated, TRUE);
+
+  for (i = 0 ; i < contact_handles->len ; i++)
+    {
+      TpHandle handle;
+
+      handle = g_array_index (contact_handles, TpHandle, i);
+      gabble_presence_cache_maybe_remove (cache, handle);
+    }
+
+}
+
 void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
     const gchar *node, GabblePresenceCapabilities new_caps)
 {
@@ -1285,7 +1466,7 @@ gabble_presence_cache_really_remove (
 {
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   const gchar *jid;
 
   jid = tp_handle_inspect (contact_repo, handle);
