@@ -18,23 +18,22 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "presence.h"
 
 #include <string.h>
-#include <glib.h>
-#include <time.h>
 
 #include "presence-cache.h"
 #include "namespaces.h"
 #include "util.h"
 
-#include "config.h"
 #define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
+
 #include "debug.h"
 
 G_DEFINE_TYPE (GabblePresence, gabble_presence, G_TYPE_OBJECT);
 
-#define GABBLE_PRESENCE_PRIV(account) ((GabblePresencePrivate *)account->priv)
+#define GABBLE_PRESENCE_PRIV(account) ((account)->priv)
 
 typedef struct _Resource Resource;
 
@@ -48,8 +47,6 @@ struct _Resource {
     time_t last_activity;
 };
 
-typedef struct _GabblePresencePrivate GabblePresencePrivate;
-
 struct _GabblePresencePrivate {
     gchar *no_resource_status_message;
     GSList *resources;
@@ -107,10 +104,10 @@ gabble_presence_init (GabblePresence *self)
 {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
       GABBLE_TYPE_PRESENCE, GabblePresencePrivate);
-  ((GabblePresencePrivate *)self->priv)->resources = NULL;
+  ((GabblePresencePrivate *) self->priv)->resources = NULL;
 }
 
-GabblePresence*
+GabblePresence *
 gabble_presence_new (void)
 {
   return g_object_new (GABBLE_TYPE_PRESENCE, NULL);
@@ -286,8 +283,7 @@ gabble_presence_update (GabblePresence *presence,
   res = _find_resource (presence, resource);
 
   /* remove, create or update a Resource as appropriate */
-  if (GABBLE_PRESENCE_OFFLINE == status &&
-      NULL == status_message)
+  if (status <= GABBLE_PRESENCE_LAST_UNAVAILABLE)
     {
       if (NULL != res)
         {
@@ -368,12 +364,14 @@ OUT:
 }
 
 LmMessage *
-gabble_presence_as_message (GabblePresence *presence, const gchar *resource)
+gabble_presence_as_message (GabblePresence *presence,
+                            const gchar *to)
 {
+  GabblePresencePrivate *priv = GABBLE_PRESENCE_PRIV (presence);
   LmMessage *message;
   LmMessageNode *node, *subnode;
   LmMessageSubType subtype;
-  Resource *res = _find_resource (presence, resource);
+  Resource *res = priv->resources->data; /* pick first resource */
 
   g_assert (NULL != res);
 
@@ -382,7 +380,7 @@ gabble_presence_as_message (GabblePresence *presence, const gchar *resource)
   else
     subtype = LM_MESSAGE_SUB_TYPE_AVAILABLE;
 
-  message = lm_message_new_with_sub_type (NULL, LM_MESSAGE_TYPE_PRESENCE,
+  message = lm_message_new_with_sub_type (to, LM_MESSAGE_TYPE_PRESENCE,
               subtype);
   node = lm_message_get_node (message);
 
@@ -410,7 +408,7 @@ gabble_presence_as_message (GabblePresence *presence, const gchar *resource)
     }
 
   if (presence->status_message)
-      lm_message_node_add_child (node, "status", presence->status_message);
+    lm_message_node_add_child (node, "status", presence->status_message);
 
   if (res->priority)
     {
