@@ -24,8 +24,8 @@
 
 #include <glib-object.h>
 
-#include "gabble-connection.h"
-#include "gabble-types.h"
+#include "connection.h"
+#include "types.h"
 
 G_BEGIN_DECLS
 
@@ -51,6 +51,8 @@ G_BEGIN_DECLS
     (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                   GABBLE_TYPE_PRESENCE, GabblePresenceClass))
 
+typedef struct _GabblePresencePrivate GabblePresencePrivate;
+
 struct _GabblePresence {
     GObject parent;
     GabblePresenceCapabilities caps;
@@ -59,7 +61,7 @@ struct _GabblePresence {
     gchar *nickname;
     gchar *avatar_sha1;
     gboolean keep_unavailable;
-    gpointer priv;
+    GabblePresencePrivate *priv;
 };
 
 typedef struct _GabblePresenceClass GabblePresenceClass;
@@ -87,7 +89,7 @@ gboolean gabble_presence_resource_has_caps (GabblePresence *presence,
                                    GabblePresenceCapabilities caps);
 
 LmMessage *gabble_presence_as_message (GabblePresence *presence,
-    const gchar *resource);
+    const gchar *to);
 gchar *gabble_presence_dump (GabblePresence *presence);
 
 G_END_DECLS
