@@ -17,34 +17,29 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "private-tubes-factory.h"
 
-#include <stdlib.h>
 #include <string.h>
-#include <time.h>
-
-#include <glib.h>
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-
 #include <loudmouth/loudmouth.h>
+#include <telepathy-glib/channel-factory-iface.h>
+#include <telepathy-glib/interfaces.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_TUBES
 
+#include "connection.h"
 #include "debug.h"
-#include "extensions/extensions.h"
-#include "gabble-connection.h"
-#include "tubes-channel.h"
-#include "namespaces.h"
-#include "util.h"
+#include "muc-channel.h"
 #include "muc-factory.h"
-#include "gabble-muc-channel.h"
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/channel-factory-iface.h>
+#include "namespaces.h"
+#include "tubes-channel.h"
+#include "util.h"
 
 static GabbleTubesChannel *new_tubes_channel (GabblePrivateTubesFactory *fac,
-    TpHandle handle);
+    TpHandle handle, TpHandle initiator);
 
 static void tubes_channel_closed_cb (GabbleTubesChannel *chan,
     gpointer user_data);
@@ -69,8 +64,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef struct _GabblePrivateTubesFactoryPrivate \
-          GabblePrivateTubesFactoryPrivate;
 struct _GabblePrivateTubesFactoryPrivate
 {
   GabbleConnection *conn;
@@ -81,8 +74,7 @@ struct _GabblePrivateTubesFactoryPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_PRIVATE_TUBES_FACTORY_GET_PRIVATE(obj) \
-    ((GabblePrivateTubesFactoryPrivate *) obj->priv)
+#define GABBLE_PRIVATE_TUBES_FACTORY_GET_PRIVATE(obj) ((obj)->priv)
 
 static void
 gabble_private_tubes_factory_init (GabblePrivateTubesFactory *self)
@@ -100,10 +92,30 @@ gabble_private_tubes_factory_init (GabblePrivateTubesFactory *self)
   priv->dispose_has_run = FALSE;
 }
 
+
+static void gabble_private_tubes_factory_close_all (
+    GabblePrivateTubesFactory *fac);
+
+
+static void
+connection_status_changed_cb (GabbleConnection *conn,
+                              guint status,
+                              guint reason,
+                              GabblePrivateTubesFactory *self)
+{
+  switch (status)
+    {
+    case TP_CONNECTION_STATUS_DISCONNECTED:
+      gabble_private_tubes_factory_close_all (self);
+      break;
+    }
+}
+
+
 static GObject *
 gabble_private_tubes_factory_constructor (GType type,
-                                  guint n_props,
-                                  GObjectConstructParam *props)
+                                          guint n_props,
+                                          GObjectConstructParam *props)
 {
   GObject *obj;
   GabblePrivateTubesFactory *self;
@@ -120,9 +132,15 @@ gabble_private_tubes_factory_constructor (GType type,
   lm_connection_register_message_handler (priv->conn->lmconn,
       priv->msg_tube_cb, LM_MESSAGE_TYPE_MESSAGE, LM_HANDLER_PRIORITY_FIRST);
 
+  /* conn is guaranteed to live longer than the factory, so this
+   * never needs disconnecting */
+  g_signal_connect (priv->conn, "status-changed",
+      (GCallback) connection_status_changed_cb, obj);
+
   return obj;
 }
 
+
 static void
 gabble_private_tubes_factory_dispose (GObject *object)
 {
@@ -136,7 +154,7 @@ gabble_private_tubes_factory_dispose (GObject *object)
   DEBUG ("dispose called");
   priv->dispose_has_run = TRUE;
 
-  tp_channel_factory_iface_close_all (TP_CHANNEL_FACTORY_IFACE (object));
+  gabble_private_tubes_factory_close_all (fac);
   g_assert (priv->channels == NULL);
 
   lm_connection_unregister_message_handler (priv->conn->lmconn,
@@ -254,7 +272,8 @@ tubes_channel_closed_cb (GabbleTubesChannel *chan,
  */
 static GabbleTubesChannel *
 new_tubes_channel (GabblePrivateTubesFactory *fac,
-                   TpHandle handle)
+                   TpHandle handle,
+                   TpHandle initiator)
 {
   GabblePrivateTubesFactoryPrivate *priv;
   TpBaseConnection *conn;
@@ -262,6 +281,8 @@ new_tubes_channel (GabblePrivateTubesFactory *fac,
   char *object_path;
 
   g_assert (GABBLE_IS_PRIVATE_TUBES_FACTORY (fac));
+  g_assert (handle != 0);
+  g_assert (initiator != 0);
 
   priv = GABBLE_PRIVATE_TUBES_FACTORY_GET_PRIVATE (fac);
   conn = (TpBaseConnection *) priv->conn;
@@ -274,6 +295,7 @@ new_tubes_channel (GabblePrivateTubesFactory *fac,
                        "object-path", object_path,
                        "handle", handle,
                        "handle-type", TP_HANDLE_TYPE_CONTACT,
+                       "initiator-handle", initiator,
                        NULL);
 
   DEBUG ("object path %s", object_path);
@@ -288,9 +310,8 @@ new_tubes_channel (GabblePrivateTubesFactory *fac,
 }
 
 static void
-gabble_private_tubes_factory_iface_close_all (TpChannelFactoryIface *iface)
+gabble_private_tubes_factory_close_all (GabblePrivateTubesFactory *fac)
 {
-  GabblePrivateTubesFactory *fac = GABBLE_PRIVATE_TUBES_FACTORY (iface);
   GabblePrivateTubesFactoryPrivate *priv =
     GABBLE_PRIVATE_TUBES_FACTORY_GET_PRIVATE (fac);
   GHashTable *tmp;
@@ -305,24 +326,6 @@ gabble_private_tubes_factory_iface_close_all (TpChannelFactoryIface *iface)
   g_hash_table_destroy (tmp);
 }
 
-static void
-gabble_private_tubes_factory_iface_connecting (TpChannelFactoryIface *iface)
-{
-  /* nothing to do */
-}
-
-static void
-gabble_private_tubes_factory_iface_connected (TpChannelFactoryIface *iface)
-{
-  /* nothing to do */
-}
-
-static void
-gabble_private_tubes_factory_iface_disconnected (TpChannelFactoryIface *iface)
-{
-  /* nothing to do */
-}
-
 struct _ForeachData
 {
   TpChannelFunc foreach;
@@ -368,8 +371,9 @@ gabble_private_tubes_factory_iface_request (TpChannelFactoryIface *iface,
   GabblePrivateTubesFactory *fac = GABBLE_PRIVATE_TUBES_FACTORY (iface);
   GabblePrivateTubesFactoryPrivate *priv =
     GABBLE_PRIVATE_TUBES_FACTORY_GET_PRIVATE (fac);
+  TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contacts_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+      base_conn, TP_HANDLE_TYPE_CONTACT);
   GabbleTubesChannel *chan;
   TpChannelFactoryRequestStatus status;
 
@@ -383,7 +387,7 @@ gabble_private_tubes_factory_iface_request (TpChannelFactoryIface *iface,
     return TP_CHANNEL_FACTORY_REQUEST_STATUS_INVALID_HANDLE;
 
   /* Don't support opening a channel to our self handle */
-  if (handle == ((TpBaseConnection*) priv->conn)->self_handle)
+  if (handle == ((TpBaseConnection *) priv->conn)->self_handle)
     {
      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
          "Can't open a channel to your self handle");
@@ -396,8 +400,8 @@ gabble_private_tubes_factory_iface_request (TpChannelFactoryIface *iface,
   if (chan == NULL)
     {
       status = TP_CHANNEL_FACTORY_REQUEST_STATUS_CREATED;
-      chan = new_tubes_channel (fac, handle);
-      tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *)chan,
+      chan = new_tubes_channel (fac, handle, base_conn->self_handle);
+      tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *) chan,
           request);
     }
 
@@ -426,7 +430,7 @@ gabble_private_tubes_factory_handle_si_tube_request (
   chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle));
   if (chan == NULL)
     {
-      chan = new_tubes_channel (self, handle);
+      chan = new_tubes_channel (self, handle, handle);
       tp_channel_factory_iface_emit_new_channel (self,
           (TpChannelIface *) chan, NULL);
 
@@ -515,7 +519,7 @@ private_tubes_factory_msg_tube_cb (LmMessageHandler *handler,
         {
           /* We create the tubes channel only if the message is a new tube
            * offer */
-          chan = new_tubes_channel (self, handle);
+          chan = new_tubes_channel (self, handle, handle);
           tp_channel_factory_iface_emit_new_channel (self,
               (TpChannelIface *) chan, NULL);
         }
@@ -549,10 +553,8 @@ gabble_private_tubes_factory_iface_init (gpointer g_iface,
 {
   TpChannelFactoryIfaceClass *klass = (TpChannelFactoryIfaceClass *) g_iface;
 
-  klass->close_all = gabble_private_tubes_factory_iface_close_all;
-  klass->connecting = gabble_private_tubes_factory_iface_connecting;
-  klass->connected = gabble_private_tubes_factory_iface_connected;
-  klass->disconnected = gabble_private_tubes_factory_iface_disconnected;
+  klass->close_all =
+      (TpChannelFactoryIfaceProc) gabble_private_tubes_factory_close_all;
   klass->foreach = gabble_private_tubes_factory_iface_foreach;
   klass->request = gabble_private_tubes_factory_iface_request;
 }
