@@ -18,13 +18,15 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "request-pipeline.h"
 
 #include <telepathy-glib/dbus.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_PIPELINE
+
+#include "connection.h"
 #include "debug.h"
-#include "gabble-connection.h"
 
 #define DEFAULT_REQUEST_TIMEOUT 20000
 #define REQUEST_PIPELINE_SIZE 5
@@ -51,7 +53,6 @@ struct _GabbleRequestPipelineItem
   gpointer user_data;
 };
 
-typedef struct _GabbleRequestPipelinePrivate GabbleRequestPipelinePrivate;
 struct _GabbleRequestPipelinePrivate
 {
   GabbleConnection *connection;
@@ -70,8 +71,7 @@ gabble_request_pipeline_error_quark (void)
   return quark;
 }
 
-#define GABBLE_REQUEST_PIPELINE_GET_PRIVATE(o) \
-  ((GabbleRequestPipelinePrivate *) ((o)->priv))
+#define GABBLE_REQUEST_PIPELINE_GET_PRIVATE(o) ((o)->priv)
 
 static void
 gabble_request_pipeline_init (GabbleRequestPipeline *obj)
