@@ -19,26 +19,26 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "roster.h"
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 
-#include <dbus/dbus-glib.h>
 #include <string.h>
 
+#include <dbus/dbus-glib.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/channel-factory-iface.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_ROSTER
 
 #include "conn-aliasing.h"
+#include "connection.h"
 #include "debug.h"
-#include "gabble-connection.h"
-#include "gabble-roster-channel.h"
 #include "namespaces.h"
-#include "util.h"
-
 #include "presence-cache.h"
+#include "roster-channel.h"
+#include "util.h"
 
 #define GOOGLE_ROSTER_VERSION "2"
 
@@ -58,7 +58,6 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-typedef struct _GabbleRosterPrivate GabbleRosterPrivate;
 struct _GabbleRosterPrivate
 {
   GabbleConnection *conn;
@@ -124,12 +123,13 @@ static void gabble_roster_get_property (GObject *object, guint property_id,
 
 static void _gabble_roster_item_free (GabbleRosterItem *item);
 static void item_edit_free (GabbleRosterItemEdit *edits);
+static void gabble_roster_close_all (GabbleRoster *roster);
 
 G_DEFINE_TYPE_WITH_CODE (GabbleRoster, gabble_roster, G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE,
       gabble_roster_factory_iface_init));
 
-#define GABBLE_ROSTER_GET_PRIVATE(o)     ((GabbleRosterPrivate*) ((o)->priv))
+#define GABBLE_ROSTER_GET_PRIVATE(o) ((o)->priv)
 
 static void
 gabble_roster_class_init (GabbleRosterClass *gabble_roster_class)
@@ -186,20 +186,6 @@ gabble_roster_init (GabbleRoster *obj)
       NULL, (GDestroyNotify) _gabble_roster_item_free);
 }
 
-static GObject *
-gabble_roster_constructor (GType type, guint n_props,
-                           GObjectConstructParam *props)
-{
-  GObject *obj;
-  /* GabbleRosterPrivate *priv; */
-
-  obj = G_OBJECT_CLASS (gabble_roster_parent_class)->
-           constructor (type, n_props, props);
-  /* priv = GABBLE_ROSTER_GET_PRIVATE (GABBLE_ROSTER (obj)); */
-
-  return obj;
-}
-
 void
 gabble_roster_dispose (GObject *object)
 {
@@ -216,7 +202,7 @@ gabble_roster_dispose (GObject *object)
   g_assert (priv->iq_cb == NULL);
   g_assert (priv->presence_cb == NULL);
 
-  tp_channel_factory_iface_close_all (TP_CHANNEL_FACTORY_IFACE (object));
+  gabble_roster_close_all (self);
   g_assert (priv->group_channels == NULL);
   g_assert (priv->list_channels == NULL);
 
@@ -230,7 +216,7 @@ item_handle_unref_foreach (gpointer key, gpointer data, gpointer user_data)
   TpHandle handle = GPOINTER_TO_UINT (key);
   GabbleRosterPrivate *priv = (GabbleRosterPrivate *) user_data;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   tp_handle_unref (contact_repo, handle);
 }
@@ -455,9 +441,9 @@ _gabble_roster_item_get (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_GROUP);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_GROUP);
   GabbleRosterItem *item;
 
   g_assert (roster != NULL);
@@ -497,7 +483,7 @@ _gabble_roster_item_remove (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   g_assert (roster != NULL);
   g_assert (GABBLE_IS_ROSTER (roster));
@@ -556,7 +542,7 @@ group_mem_update_ensure (GroupsUpdateContext *ctx,
 static void
 _update_add_to_group (guint group_handle, gpointer user_data)
 {
-  GroupsUpdateContext *ctx = (GroupsUpdateContext *)user_data;
+  GroupsUpdateContext *ctx = (GroupsUpdateContext *) user_data;
   GroupMembershipUpdate *update = group_mem_update_ensure (ctx, group_handle);
 
   DEBUG ("- contact#%u added to group#%u", ctx->contact_handle,
@@ -567,7 +553,7 @@ _update_add_to_group (guint group_handle, gpointer user_data)
 static void
 _update_remove_from_group (guint group_handle, gpointer user_data)
 {
-  GroupsUpdateContext *ctx = (GroupsUpdateContext *)user_data;
+  GroupsUpdateContext *ctx = (GroupsUpdateContext *) user_data;
   GroupMembershipUpdate *update = group_mem_update_ensure (ctx, group_handle);
 
   DEBUG ("- contact#%u removed from group#%u", ctx->contact_handle,
@@ -648,7 +634,7 @@ _gabble_roster_item_update (GabbleRoster *roster,
     }
 
   new_groups_handle_set = _parse_item_groups (node,
-      (TpBaseConnection *)priv->conn);
+      (TpBaseConnection *) priv->conn);
   new_groups = tp_handle_set_peek (new_groups_handle_set);
 
   removed_from = tp_intset_difference (tp_handle_set_peek (item->groups),
@@ -677,7 +663,7 @@ _gabble_roster_item_update (GabbleRoster *roster,
 static void
 _gabble_roster_item_dump_group (guint handle, gpointer user_data)
 {
-  g_string_append_printf ((GString *)user_data, "group#%u ", handle);
+  g_string_append_printf ((GString *) user_data, "group#%u ", handle);
 }
 
 static gchar *
@@ -782,12 +768,12 @@ _gabble_roster_item_to_message (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   LmMessage *message;
   LmMessageNode *query_node, *item_node;
   const gchar *jid;
   struct _ItemToMessageContext ctx = {
-      (TpBaseConnection *)priv->conn,
+      (TpBaseConnection *) priv->conn,
   };
 
   g_assert (roster != NULL);
@@ -846,7 +832,7 @@ _gabble_roster_create_channel (GabbleRoster *roster,
                                TpHandle handle)
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (conn,
       handle_type);
   GabbleRosterChannel *chan;
@@ -891,7 +877,7 @@ _gabble_roster_create_channel (GabbleRoster *roster,
              object_path);
 
       tp_channel_factory_iface_emit_new_channel (roster,
-          (TpChannelIface *)chan, NULL);
+          (TpChannelIface *) chan, NULL);
     }
   else
     {
@@ -911,7 +897,7 @@ _gabble_roster_get_channel (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, handle_type);
+      (TpBaseConnection *) priv->conn, handle_type);
   GabbleRosterChannel *chan;
   GHashTable *channels = (handle_type == TP_HANDLE_TYPE_LIST
                           ? priv->list_channels
@@ -958,7 +944,7 @@ _gabble_roster_emit_one (gpointer key,
 #ifdef ENABLE_DEBUG
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, data_struct->handle_type);
+      (TpBaseConnection *) priv->conn, data_struct->handle_type);
   TpHandle handle = GPOINTER_TO_INT (key);
   const gchar *name;
 
@@ -971,7 +957,7 @@ _gabble_roster_emit_one (gpointer key,
       name);
 #endif
 
-  tp_channel_factory_iface_emit_new_channel (roster, (TpChannelIface *)chan,
+  tp_channel_factory_iface_emit_new_channel (roster, (TpChannelIface *) chan,
       NULL);
 }
 
@@ -1048,7 +1034,7 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
 {
   GabbleRoster *roster = GABBLE_ROSTER (user_data);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
   LmMessageNode *iq_node, *query_node;
@@ -1329,7 +1315,7 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
           NULL);
 
       DEBUG ("calling change members on known channel");
-      tp_group_mixin_change_members ((GObject *)chan,
+      tp_group_mixin_change_members ((GObject *) chan,
             "", known_add, known_rem, NULL, NULL, 0, 0);
 
       DEBUG ("calling change members on any group channels");
@@ -1342,7 +1328,7 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
               handle, NULL);
 
           DEBUG ("calling change members on deny channel");
-          tp_group_mixin_change_members ((GObject *)chan,
+          tp_group_mixin_change_members ((GObject *) chan,
               "", deny_add, deny_rem, NULL, NULL, 0, 0);
 
           tp_intset_destroy (deny_add);
@@ -1447,7 +1433,7 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
 {
   GabbleRoster *roster = GABBLE_ROSTER (user_data);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
   LmMessageNode *pres_node, *child_node;
@@ -1512,7 +1498,7 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
       list_handle = GABBLE_LIST_HANDLE_PUBLISH;
       chan = _gabble_roster_get_channel (roster, TP_HANDLE_TYPE_LIST,
           list_handle, NULL);
-      tp_group_mixin_change_members ((GObject *)chan, status_message,
+      tp_group_mixin_change_members ((GObject *) chan, status_message,
           NULL, NULL, tmp, NULL, 0, 0);
 
       tp_intset_destroy (tmp);
@@ -1529,7 +1515,7 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
       list_handle = GABBLE_LIST_HANDLE_PUBLISH;
       chan = _gabble_roster_get_channel (roster, TP_HANDLE_TYPE_LIST,
           list_handle, NULL);
-      changed = tp_group_mixin_change_members ((GObject *)chan,
+      changed = tp_group_mixin_change_members ((GObject *) chan,
           status_message, NULL, tmp, NULL, NULL, 0, 0);
 
       _gabble_roster_send_presence_ack (roster, from, sub_type, changed);
@@ -1548,7 +1534,7 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
       list_handle = GABBLE_LIST_HANDLE_SUBSCRIBE;
       chan = _gabble_roster_get_channel (roster, TP_HANDLE_TYPE_LIST,
           list_handle, NULL);
-      changed = tp_group_mixin_change_members ((GObject *)chan,
+      changed = tp_group_mixin_change_members ((GObject *) chan,
           status_message, tmp, NULL, NULL, NULL, 0, 0);
 
       _gabble_roster_send_presence_ack (roster, from, sub_type, changed);
@@ -1567,7 +1553,7 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
       list_handle = GABBLE_LIST_HANDLE_SUBSCRIBE;
       chan = _gabble_roster_get_channel (roster, TP_HANDLE_TYPE_LIST,
           list_handle, NULL);
-      changed = tp_group_mixin_change_members ((GObject *)chan,
+      changed = tp_group_mixin_change_members ((GObject *) chan,
           status_message, NULL, tmp, NULL, NULL, 0, 0);
 
       _gabble_roster_send_presence_ack (roster, from, sub_type, changed);
@@ -1586,10 +1572,9 @@ OUT:
 }
 
 static void
-gabble_roster_factory_iface_close_all (TpChannelFactoryIface *iface)
+gabble_roster_close_all (GabbleRoster *self)
 {
-  GabbleRoster *roster = GABBLE_ROSTER (iface);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (self);
 
   DEBUG ("closing channels");
 
@@ -1606,72 +1591,89 @@ gabble_roster_factory_iface_close_all (TpChannelFactoryIface *iface)
     }
 }
 
-static void
-gabble_roster_factory_iface_connecting (TpChannelFactoryIface *iface)
-{
-  GabbleRoster *roster = GABBLE_ROSTER (iface);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
-
-  DEBUG ("adding callbacks");
-
-  g_assert (priv->iq_cb == NULL);
-  g_assert (priv->presence_cb == NULL);
-
-  priv->iq_cb = lm_message_handler_new (gabble_roster_iq_cb,
-                                        roster, NULL);
-  lm_connection_register_message_handler (priv->conn->lmconn,
-                                          priv->iq_cb,
-                                          LM_MESSAGE_TYPE_IQ,
-                                          LM_HANDLER_PRIORITY_NORMAL);
-
-  priv->presence_cb = lm_message_handler_new (gabble_roster_presence_cb,
-                                              roster, NULL);
-  lm_connection_register_message_handler (priv->conn->lmconn,
-                                          priv->presence_cb,
-                                          LM_MESSAGE_TYPE_PRESENCE,
-                                          LM_HANDLER_PRIORITY_LAST);
-}
 
 static void
-gabble_roster_factory_iface_connected (TpChannelFactoryIface *iface)
+connection_status_changed_cb (GabbleConnection *conn,
+                              guint status,
+                              guint reason,
+                              GabbleRoster *self)
 {
-  GabbleRoster *roster = GABBLE_ROSTER (iface);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
-  LmMessage *message;
+  switch (status)
+    {
+    case TP_CONNECTION_STATUS_CONNECTING:
+      DEBUG ("adding callbacks");
+      g_assert (self->priv->iq_cb == NULL);
+      g_assert (self->priv->presence_cb == NULL);
 
-  DEBUG ("requesting roster");
+      self->priv->iq_cb = lm_message_handler_new (gabble_roster_iq_cb,
+          self, NULL);
+      lm_connection_register_message_handler (self->priv->conn->lmconn,
+          self->priv->iq_cb, LM_MESSAGE_TYPE_IQ,
+          LM_HANDLER_PRIORITY_NORMAL);
 
-  message = _gabble_roster_message_new (roster, LM_MESSAGE_SUB_TYPE_GET, NULL);
+      self->priv->presence_cb = lm_message_handler_new (
+          gabble_roster_presence_cb, self, NULL);
+      lm_connection_register_message_handler (self->priv->conn->lmconn,
+          self->priv->presence_cb, LM_MESSAGE_TYPE_PRESENCE,
+          LM_HANDLER_PRIORITY_LAST);
 
-  _gabble_connection_send (priv->conn, message, NULL);
+      break;
 
-  lm_message_unref (message);
+    case TP_CONNECTION_STATUS_CONNECTED:
+        {
+          LmMessage *message;
+
+          DEBUG ("requesting roster");
+
+          message = _gabble_roster_message_new (self, LM_MESSAGE_SUB_TYPE_GET,
+              NULL);
+          _gabble_connection_send (self->priv->conn, message, NULL);
+          lm_message_unref (message);
+        }
+      break;
+
+    case TP_CONNECTION_STATUS_DISCONNECTED:
+      gabble_roster_close_all (self);
+
+      /* this can be called before we have ever been CONNECTING, so we need
+       * to guard it */
+      if (self->priv->iq_cb != NULL)
+        {
+          DEBUG ("removing callbacks");
+          g_assert (self->priv->presence_cb != NULL);
+
+          lm_connection_unregister_message_handler (self->priv->conn->lmconn,
+              self->priv->iq_cb, LM_MESSAGE_TYPE_IQ);
+          lm_message_handler_unref (self->priv->iq_cb);
+          self->priv->iq_cb = NULL;
+
+          lm_connection_unregister_message_handler (self->priv->conn->lmconn,
+              self->priv->presence_cb, LM_MESSAGE_TYPE_PRESENCE);
+          lm_message_handler_unref (self->priv->presence_cb);
+          self->priv->presence_cb = NULL;
+        }
+      break;
+    }
 }
 
-static void
-gabble_roster_factory_iface_disconnected (TpChannelFactoryIface *iface)
+
+static GObject *
+gabble_roster_constructor (GType type, guint n_props,
+                           GObjectConstructParam *props)
 {
-  GabbleRoster *roster = GABBLE_ROSTER (iface);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GObject *obj = G_OBJECT_CLASS (gabble_roster_parent_class)->
+           constructor (type, n_props, props);
+  GabbleRoster *self = GABBLE_ROSTER (obj);
 
-  DEBUG ("removing callbacks");
+  /* conn is guaranteed to live longer than the factory, so this
+   * never needs disconnecting */
+  g_signal_connect (self->priv->conn, "status-changed",
+      (GCallback) connection_status_changed_cb, obj);
 
-  g_assert (priv->iq_cb != NULL);
-  g_assert (priv->presence_cb != NULL);
-
-  lm_connection_unregister_message_handler (priv->conn->lmconn,
-                                            priv->iq_cb,
-                                            LM_MESSAGE_TYPE_IQ);
-  lm_message_handler_unref (priv->iq_cb);
-  priv->iq_cb = NULL;
-
-  lm_connection_unregister_message_handler (priv->conn->lmconn,
-                                            priv->presence_cb,
-                                            LM_MESSAGE_TYPE_PRESENCE);
-  lm_message_handler_unref (priv->presence_cb);
-  priv->presence_cb = NULL;
+  return obj;
 }
 
+
 struct foreach_data {
     TpChannelFunc func;
     gpointer data;
@@ -1718,7 +1720,7 @@ gabble_roster_factory_iface_request (TpChannelFactoryIface *iface,
   GabbleRoster *roster = GABBLE_ROSTER (iface);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, handle_type);
+      (TpBaseConnection *) priv->conn, handle_type);
   gboolean created;
   GabbleRosterChannel *chan;
 
@@ -1758,10 +1760,7 @@ gabble_roster_factory_iface_init (gpointer g_iface,
 {
   TpChannelFactoryIfaceClass *klass = (TpChannelFactoryIfaceClass *) g_iface;
 
-  klass->close_all = gabble_roster_factory_iface_close_all;
-  klass->connecting = gabble_roster_factory_iface_connecting;
-  klass->connected = gabble_roster_factory_iface_connected;
-  klass->disconnected = gabble_roster_factory_iface_disconnected;
+  klass->close_all = (TpChannelFactoryIfaceProc) gabble_roster_close_all;
   klass->foreach = gabble_roster_factory_iface_foreach;
   klass->request = gabble_roster_factory_iface_request;
 }
@@ -1821,7 +1820,7 @@ roster_item_apply_edits (GabbleRoster *roster,
   TpIntSet *intset;
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_GROUP);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_GROUP);
   GabbleRosterItemEdit *edits = item->unsent_edits;
   LmMessage *message;
 
@@ -1991,7 +1990,7 @@ gabble_roster_handle_get_subscription (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
 
   g_return_val_if_fail (roster != NULL, GABBLE_ROSTER_SUBSCRIPTION_NONE);
@@ -2016,7 +2015,7 @@ gabble_roster_handle_set_blocked (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
   GoogleItemType orig_type;
   LmMessage *message;
@@ -2083,7 +2082,7 @@ gabble_roster_handle_has_entry (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
 
   g_return_val_if_fail (roster != NULL, FALSE);
@@ -2102,7 +2101,7 @@ gabble_roster_handle_get_name (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
 
   g_return_val_if_fail (roster != NULL, NULL);
@@ -2126,7 +2125,7 @@ gabble_roster_handle_set_name (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
   LmMessage *message;
   LmMessageNode *item_node;
@@ -2179,7 +2178,7 @@ gabble_roster_handle_remove (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
   GabbleRosterSubscription subscription;
   LmMessage *message;
@@ -2230,7 +2229,7 @@ gabble_roster_handle_add (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
   LmMessage *message;
   gboolean do_add = FALSE;
@@ -2288,9 +2287,9 @@ gabble_roster_handle_add_to_group (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_GROUP);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_GROUP);
   GabbleRosterItem *item;
   LmMessage *message;
   gboolean ret;
@@ -2348,9 +2347,9 @@ gabble_roster_handle_remove_from_group (GabbleRoster *roster,
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_GROUP);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_GROUP);
   GabbleRosterItem *item;
   LmMessage *message;
   gboolean ret, was_in_group;
