@@ -23,21 +23,21 @@
 
 #define _GNU_SOURCE /* Needed for strptime (_XOPEN_SOURCE can also be used). */
 
+#include "config.h"
 #include "text-mixin.h"
 
-#include <loudmouth/loudmouth.h>
-#include <dbus/dbus-glib.h>
 #include <string.h>
-#include <time.h>
 
+#include <dbus/dbus-glib.h>
+#include <loudmouth/loudmouth.h>
 #include <telepathy-glib/text-mixin.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/errors.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_IM
 
+#include "connection.h"
 #include "debug.h"
-#include "gabble-connection.h"
 #include "namespaces.h"
 #include "roster.h"
 #include "util.h"
