@@ -17,35 +17,30 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "tube-dbus.h"
 
-#include <stdlib.h>
 #include <string.h>
-#include <time.h>
 #include <errno.h>
 
-#include <glib.h>
 #include <glib/gstdio.h>
-
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-
 #include <loudmouth/loudmouth.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_TUBES
 
+#include "base64.h"
+#include "bytestream-factory.h"
+#include "bytestream-ibb.h"
+#include "bytestream-iface.h"
+#include "connection.h"
 #include "debug.h"
 #include "disco.h"
-#include "extensions/extensions.h"
-#include "gabble-connection.h"
-#include "namespaces.h"
-#include "util.h"
-#include "base64.h"
-#include "tube-iface.h"
-#include "bytestream-iface.h"
-#include "bytestream-ibb.h"
-#include "bytestream-factory.h"
 #include "gabble-signals-marshal.h"
+#include "namespaces.h"
+#include "tube-iface.h"
+#include "util.h"
 
 /* When we receive D-Bus messages to be delivered to the application and the
  * application is not yet connected to the D-Bus tube, theses D-Bus messages
@@ -93,7 +88,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef struct _GabbleTubeDBusPrivate GabbleTubeDBusPrivate;
 struct _GabbleTubeDBusPrivate
 {
   GabbleConnection *conn;
@@ -135,8 +129,7 @@ struct _GabbleTubeDBusPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_TUBE_DBUS_GET_PRIVATE(obj) \
-    ((GabbleTubeDBusPrivate *) obj->priv)
+#define GABBLE_TUBE_DBUS_GET_PRIVATE(obj) ((obj)->priv)
 
 static void data_received_cb (GabbleBytestreamIface *stream, TpHandle sender,
     GString *data, gpointer user_data);
