@@ -17,82 +17,91 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "tubes-channel.h"
 
-#include <dbus/dbus-glib.h>
-#include <stdio.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 #include <unistd.h>
-#include <errno.h>
 
-#include <glib.h>
 #include <glib/gstdio.h>
-
+#include <dbus/dbus-glib.h>
 #include <telepathy-glib/channel-iface.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/group-mixin.h>
+#include <telepathy-glib/gtypes.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/svc-channel.h>
+#include <telepathy-glib/svc-generic.h>
+
+#include "extensions/extensions.h"
 
 #define DEBUG_FLAG GABBLE_DEBUG_TUBES
 
+#include "bytestream-factory.h"
+#include "connection.h"
 #include "debug.h"
-#include "gabble-connection.h"
-#include "presence.h"
-#include "presence-cache.h"
+#include "exportable-channel.h"
 #include "namespaces.h"
-#include "util.h"
+#include "presence-cache.h"
+#include "presence.h"
 #include "tube-iface.h"
 #include "tube-stream.h"
-#include "bytestream-factory.h"
+#include "util.h"
 
 #ifdef HAVE_DBUS_TUBE
 #include "tube-dbus.h"
 #endif
 
-#define GABBLE_CHANNEL_TUBE_TYPE \
-    (dbus_g_type_get_struct ("GValueArray", \
-        G_TYPE_UINT, \
-        G_TYPE_UINT, \
-        G_TYPE_UINT, \
-        G_TYPE_STRING, \
-        dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), \
-        G_TYPE_UINT, \
-        G_TYPE_INVALID))
-
-#define DBUS_NAME_PAIR_TYPE \
-    (dbus_g_type_get_struct ("GValueArray", \
-      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID))
-
 static void channel_iface_init (gpointer, gpointer);
 static void tubes_iface_init (gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE (GabbleTubesChannel, gabble_tubes_channel,
     G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
+      tp_dbus_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
+    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_CHANNEL_FUTURE, NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_TUBES, tubes_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_GROUP,
         tp_external_group_mixin_iface_init);
+    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_EXPORTABLE_CHANNEL, NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_IFACE, NULL));
 
+static const gchar *gabble_tubes_channel_interfaces[] = {
+    TP_IFACE_CHANNEL_INTERFACE_GROUP,
+    /* If more interfaces are added, either keep Group as the first, or change
+     * the implementations of gabble_tubes_channel_get_interfaces () and
+     * gabble_tubes_channel_get_property () too */
+    GABBLE_IFACE_CHANNEL_FUTURE,
+    NULL
+};
+
+
 enum
 {
   PROP_OBJECT_PATH = 1,
   PROP_CHANNEL_TYPE,
   PROP_HANDLE_TYPE,
   PROP_HANDLE,
+  PROP_TARGET_ID,
+  PROP_REQUESTED,
   PROP_CONNECTION,
+  PROP_INTERFACES,
   PROP_MUC,
+  PROP_INITIATOR_HANDLE,
+  PROP_INITIATOR_ID,
+  PROP_CHANNEL_DESTROYED,
+  PROP_CHANNEL_PROPERTIES,
   LAST_PROPERTY,
 };
 
 /* private structure */
-typedef struct _GabbleTubesChannelPrivate GabbleTubesChannelPrivate;
 
 struct _GabbleTubesChannelPrivate
 {
@@ -101,6 +110,7 @@ struct _GabbleTubesChannelPrivate
   TpHandle handle;
   TpHandleType handle_type;
   TpHandle self_handle;
+  TpHandle initiator;
 
   GHashTable *tubes;
 
@@ -109,8 +119,7 @@ struct _GabbleTubesChannelPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_TUBES_CHANNEL_GET_PRIVATE(obj) \
-    ((GabbleTubesChannelPrivate *) obj->priv)
+#define GABBLE_TUBES_CHANNEL_GET_PRIVATE(obj) ((obj)->priv)
 
 static gboolean update_tubes_presence (GabbleTubesChannel *self);
 
@@ -138,7 +147,7 @@ gabble_tubes_channel_constructor (GType type,
   GabbleTubesChannel *self;
   GabbleTubesChannelPrivate *priv;
   DBusGConnection *bus;
-  TpHandleRepoIface *handle_repo;
+  TpHandleRepoIface *handle_repo, *contact_repo;
 
   DEBUG ("Called");
 
@@ -147,15 +156,21 @@ gabble_tubes_channel_constructor (GType type,
 
   self = GABBLE_TUBES_CHANNEL (obj);
   priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
+  contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   handle_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, priv->handle_type);
 
   tp_handle_ref (handle_repo, priv->handle);
 
+  if (priv->initiator != 0)
+    tp_handle_ref (contact_repo, priv->initiator);
+
   switch (priv->handle_type)
     {
     case TP_HANDLE_TYPE_CONTACT:
       g_assert (self->muc == NULL);
+      g_assert (priv->initiator != 0);
       priv->self_handle = ((TpBaseConnection *) (priv->conn))->self_handle;
       break;
     case TP_HANDLE_TYPE_ROOM:
@@ -187,6 +202,7 @@ gabble_tubes_channel_get_property (GObject *object,
 {
   GabbleTubesChannel *chan = GABBLE_TUBES_CHANNEL (object);
   GabbleTubesChannelPrivate *priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (chan);
+  TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
 
   switch (property_id)
     {
@@ -202,12 +218,69 @@ gabble_tubes_channel_get_property (GObject *object,
       case PROP_HANDLE:
         g_value_set_uint (value, priv->handle);
         break;
+      case PROP_TARGET_ID:
+          {
+            TpHandleRepoIface *repo = tp_base_connection_get_handles (
+                base_conn, priv->handle_type);
+
+            g_value_set_string (value,
+                tp_handle_inspect (repo, priv->handle));
+          }
+        break;
       case PROP_CONNECTION:
         g_value_set_object (value, priv->conn);
         break;
+      case PROP_INTERFACES:
+        if (chan->muc)
+          {
+            /* MUC tubes */
+            g_value_set_boxed (value, gabble_tubes_channel_interfaces);
+          }
+        else
+          {
+            /* 1-1 tubes - omit the Group interface */
+            g_value_set_boxed (value, gabble_tubes_channel_interfaces + 1);
+          }
+        break;
       case PROP_MUC:
         g_value_set_object (value, chan->muc);
         break;
+      case PROP_INITIATOR_HANDLE:
+        g_value_set_uint (value, priv->initiator);
+        break;
+      case PROP_INITIATOR_ID:
+        if (priv->initiator == 0)
+          {
+            g_value_set_static_string (value, "");
+          }
+        else
+          {
+            TpHandleRepoIface *repo = tp_base_connection_get_handles (
+                base_conn, TP_HANDLE_TYPE_CONTACT);
+
+            g_value_set_string (value,
+                tp_handle_inspect (repo, priv->initiator));
+          }
+        break;
+      case PROP_REQUESTED:
+        g_value_set_boolean (value,
+            (priv->initiator == base_conn->self_handle));
+        break;
+      case PROP_CHANNEL_DESTROYED:
+        g_value_set_boolean (value, priv->closed);
+        break;
+      case PROP_CHANNEL_PROPERTIES:
+        g_value_set_boxed (value,
+            gabble_tp_dbus_properties_mixin_make_properties_hash (object,
+                TP_IFACE_CHANNEL, "TargetHandle",
+                TP_IFACE_CHANNEL, "TargetHandleType",
+                TP_IFACE_CHANNEL, "ChannelType",
+                TP_IFACE_CHANNEL, "TargetID",
+                GABBLE_IFACE_CHANNEL_FUTURE, "InitiatorHandle",
+                GABBLE_IFACE_CHANNEL_FUTURE, "InitiatorID",
+                GABBLE_IFACE_CHANNEL_FUTURE, "Requested",
+                NULL));
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -246,6 +319,9 @@ gabble_tubes_channel_set_property (GObject *object,
       case PROP_MUC:
         chan->muc = g_value_get_object (value);
         break;
+      case PROP_INITIATOR_HANDLE:
+        priv->initiator = g_value_get_uint (value);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -264,9 +340,9 @@ d_bus_names_changed_added (GabbleTubesChannel *self,
   GValue tmp = {0,};
   guint i;
 
-  g_value_init (&tmp, DBUS_NAME_PAIR_TYPE);
+  g_value_init (&tmp, TP_STRUCT_TYPE_DBUS_TUBE_MEMBER);
   g_value_take_boxed (&tmp,
-      dbus_g_type_specialized_construct (DBUS_NAME_PAIR_TYPE));
+      dbus_g_type_specialized_construct (TP_STRUCT_TYPE_DBUS_TUBE_MEMBER));
   dbus_g_type_struct_set (&tmp,
       0, contact,
       1, new_name,
@@ -277,7 +353,7 @@ d_bus_names_changed_added (GabbleTubesChannel *self,
       tube_id, added, removed);
 
   for (i = 0; i < added->len; i++)
-    g_boxed_free (DBUS_NAME_PAIR_TYPE, added->pdata[i]);
+    g_boxed_free (TP_STRUCT_TYPE_DBUS_TUBE_MEMBER, added->pdata[i]);
   g_ptr_array_free (added, TRUE);
   g_array_free (removed, TRUE);
 }
@@ -835,9 +911,9 @@ copy_tube_in_ptr_array (gpointer key,
                 "state", &state,
                 NULL);
 
-  g_value_init (&entry, GABBLE_CHANNEL_TUBE_TYPE);
+  g_value_init (&entry, TP_STRUCT_TYPE_TUBE_INFO);
   g_value_take_boxed (&entry,
-          dbus_g_type_specialized_construct (GABBLE_CHANNEL_TUBE_TYPE));
+          dbus_g_type_specialized_construct (TP_STRUCT_TYPE_TUBE_INFO));
   dbus_g_type_struct_set (&entry,
           0, tube_id,
           1, initiator,
@@ -918,7 +994,7 @@ gabble_tubes_channel_list_tubes (TpSvcChannelTypeTubes *iface,
   tp_svc_channel_type_tubes_return_from_list_tubes (context, ret);
 
   for (i = 0; i < ret->len; i++)
-    g_boxed_free (GABBLE_CHANNEL_TUBE_TYPE, ret->pdata[i]);
+    g_boxed_free (TP_STRUCT_TYPE_TUBE_INFO, ret->pdata[i]);
 
   g_ptr_array_free (ret, TRUE);
 }
@@ -1316,7 +1392,7 @@ start_stream_initiation (GabbleTubesChannel *self,
   priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
 
   contact_repo = tp_base_connection_get_handles (
-     (TpBaseConnection*) priv->conn, TP_HANDLE_TYPE_CONTACT);
+     (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   jid = tp_handle_inspect (contact_repo, priv->handle);
 
@@ -1398,7 +1474,7 @@ send_new_stream_tube_msg (GabbleTubesChannel *self,
   priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
 
   contact_repo = tp_base_connection_get_handles (
-     (TpBaseConnection*) priv->conn, TP_HANDLE_TYPE_CONTACT);
+     (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   jid = tp_handle_inspect (contact_repo, priv->handle);
 
@@ -1440,7 +1516,7 @@ send_tube_close_msg (GabbleTubesChannel *self,
   LmMessage *msg;
   const gchar *jid;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection*) priv->conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   gchar *id_str;
 
   jid = tp_handle_inspect (contact_repo, priv->handle);
@@ -1630,7 +1706,7 @@ gabble_tubes_channel_offer_d_bus_tube (TpSvcChannelTypeTubes *iface,
   g_assert (GABBLE_IS_TUBES_CHANNEL (self));
 
   priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
-  base = (TpBaseConnection*) priv->conn;
+  base = (TpBaseConnection *) priv->conn;
 
   parameters_copied = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
       (GDestroyNotify) tp_g_value_slice_free);
@@ -1718,7 +1794,7 @@ gabble_tubes_channel_offer_stream_tube (TpSvcChannelTypeTubes *iface,
   g_assert (GABBLE_IS_TUBES_CHANNEL (self));
 
   priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
-  base = (TpBaseConnection*) priv->conn;
+  base = (TpBaseConnection *) priv->conn;
 
   if (!gabble_tube_stream_check_params (address_type, address,
         access_control, access_control_param, &error))
@@ -2053,9 +2129,9 @@ get_d_bus_names_foreach (gpointer key,
   GPtrArray *ret = user_data;
   GValue tmp = {0,};
 
-  g_value_init (&tmp, DBUS_NAME_PAIR_TYPE);
+  g_value_init (&tmp, TP_STRUCT_TYPE_DBUS_TUBE_MEMBER);
   g_value_take_boxed (&tmp,
-      dbus_g_type_specialized_construct (DBUS_NAME_PAIR_TYPE));
+      dbus_g_type_specialized_construct (TP_STRUCT_TYPE_DBUS_TUBE_MEMBER));
   dbus_g_type_struct_set (&tmp,
       0, key,
       1, value,
@@ -2130,7 +2206,7 @@ gabble_tubes_channel_get_d_bus_names (TpSvcChannelTypeTubes *iface,
   tp_svc_channel_type_tubes_return_from_get_d_bus_names (context, ret);
 
   for (i = 0; i < ret->len; i++)
-    g_boxed_free (DBUS_NAME_PAIR_TYPE, ret->pdata[i]);
+    g_boxed_free (TP_STRUCT_TYPE_DBUS_TUBE_MEMBER, ret->pdata[i]);
   g_hash_table_unref (names);
   g_ptr_array_free (ret, TRUE);
 
@@ -2257,7 +2333,7 @@ emit_tube_closed_signal (gpointer key,
                          gpointer user_data)
 {
   guint id = GPOINTER_TO_UINT (key);
-  GabbleTubesChannel *self = (GabbleTubesChannel*) user_data;
+  GabbleTubesChannel *self = (GabbleTubesChannel *) user_data;
 
   tp_svc_channel_type_tubes_emit_tube_closed (self, id);
 }
@@ -2350,19 +2426,18 @@ static void
 gabble_tubes_channel_get_interfaces (TpSvcChannel *iface,
                                      DBusGMethodInvocation *context)
 {
-  const char *interfaces[] = {
-      TP_IFACE_CHANNEL_INTERFACE_GROUP,
-      NULL };
   GabbleTubesChannel *self = GABBLE_TUBES_CHANNEL (iface);
 
   if (self->muc)
     {
-      tp_svc_channel_return_from_get_interfaces (context, interfaces);
+      tp_svc_channel_return_from_get_interfaces (context,
+          gabble_tubes_channel_interfaces);
     }
   else
     {
-      /* only show the NULL */
-      tp_svc_channel_return_from_get_interfaces (context, interfaces + 1);
+      /* omit the Group interface */
+      tp_svc_channel_return_from_get_interfaces (context,
+          gabble_tubes_channel_interfaces + 1);
     }
 }
 
@@ -2373,6 +2448,33 @@ static void
 gabble_tubes_channel_class_init (
     GabbleTubesChannelClass *gabble_tubes_channel_class)
 {
+  static TpDBusPropertiesMixinPropImpl channel_props[] = {
+      { "TargetHandleType", "handle-type", NULL },
+      { "TargetHandle", "handle", NULL },
+      { "TargetID", "target-id", NULL },
+      { "ChannelType", "channel-type", NULL },
+      { "Interfaces", "interfaces", NULL },
+      { NULL }
+  };
+  static TpDBusPropertiesMixinPropImpl future_props[] = {
+      { "Requested", "requested", NULL },
+      { "InitiatorHandle", "initiator-handle", NULL },
+      { "InitiatorID", "initiator-id", NULL },
+      { NULL }
+  };
+  static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
+      { TP_IFACE_CHANNEL,
+        tp_dbus_properties_mixin_getter_gobject_properties,
+        NULL,
+        channel_props,
+      },
+      { GABBLE_IFACE_CHANNEL_FUTURE,
+        tp_dbus_properties_mixin_getter_gobject_properties,
+        NULL,
+        future_props,
+      },
+      { NULL }
+  };
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_tubes_channel_class);
   GParamSpec *param_spec;
 
@@ -2395,6 +2497,11 @@ gabble_tubes_channel_class_init (
       "handle-type");
   g_object_class_override_property (object_class, PROP_HANDLE, "handle");
 
+  g_object_class_override_property (object_class, PROP_CHANNEL_DESTROYED,
+      "channel-destroyed");
+  g_object_class_override_property (object_class, PROP_CHANNEL_PROPERTIES,
+      "channel-properties");
+
   param_spec = g_param_spec_object (
       "connection",
       "GabbleConnection object",
@@ -2407,6 +2514,13 @@ gabble_tubes_channel_class_init (
       G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
+  param_spec = g_param_spec_boxed ("interfaces", "Extra D-Bus interfaces",
+      "Additional Channel.Interface.* interfaces",
+      G_TYPE_STRV,
+      G_PARAM_READABLE |
+      G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME);
+  g_object_class_install_property (object_class, PROP_INTERFACES, param_spec);
+
   param_spec = g_param_spec_object (
       "muc",
       "GabbleMucChannel object",
@@ -2419,6 +2533,42 @@ gabble_tubes_channel_class_init (
       G_PARAM_STATIC_NICK |
       G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_MUC, param_spec);
+
+  param_spec = g_param_spec_string ("target-id", "Target JID",
+      "The string obtained by inspecting the target handle",
+      NULL,
+      G_PARAM_READABLE |
+      G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME);
+  g_object_class_install_property (object_class, PROP_TARGET_ID, param_spec);
+
+  param_spec = g_param_spec_uint ("initiator-handle", "Initiator's handle",
+      "The contact who initiated the channel",
+      0, G_MAXUINT32, 0,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE |
+      G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME);
+  g_object_class_install_property (object_class, PROP_INITIATOR_HANDLE,
+      param_spec);
+
+  param_spec = g_param_spec_string ("initiator-id", "Initiator's bare JID",
+      "The string obtained by inspecting the initiator-handle",
+      NULL,
+      G_PARAM_READABLE |
+      G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME);
+  g_object_class_install_property (object_class, PROP_INITIATOR_ID,
+      param_spec);
+
+  param_spec = g_param_spec_boolean ("requested", "Requested?",
+      "True if this channel was requested by the local user",
+      FALSE,
+      G_PARAM_READABLE |
+      G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME);
+  g_object_class_install_property (object_class, PROP_REQUESTED, param_spec);
+
+  gabble_tubes_channel_class->dbus_props_class.interfaces = prop_interfaces;
+  tp_dbus_properties_mixin_class_init (object_class,
+      G_STRUCT_OFFSET (GabbleTubesChannelClass, dbus_props_class));
+
+  tp_external_group_mixin_init_dbus_properties (object_class);
 }
 
 void
@@ -2456,9 +2606,14 @@ gabble_tubes_channel_finalize (GObject *object)
 {
   GabbleTubesChannel *self = GABBLE_TUBES_CHANNEL (object);
   GabbleTubesChannelPrivate *priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
+  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   g_free (priv->object_path);
 
+  if (priv->initiator != 0)
+    tp_handle_unref (contact_handles, priv->initiator);
+
   G_OBJECT_CLASS (gabble_tubes_channel_parent_class)->finalize (object);
 }
 
@@ -2466,7 +2621,7 @@ static void
 tubes_iface_init (gpointer g_iface,
                   gpointer iface_data)
 {
-  TpSvcChannelTypeTubesClass *klass = (TpSvcChannelTypeTubesClass *)g_iface;
+  TpSvcChannelTypeTubesClass *klass = (TpSvcChannelTypeTubesClass *) g_iface;
 
 #define IMPLEMENT(x) tp_svc_channel_type_tubes_implement_##x (\
     klass, gabble_tubes_channel_##x)
@@ -2488,7 +2643,7 @@ static void
 channel_iface_init (gpointer g_iface,
                     gpointer iface_data)
 {
-  TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
+  TpSvcChannelClass *klass = (TpSvcChannelClass *) g_iface;
 
 #define IMPLEMENT(x, suffix) tp_svc_channel_implement_##x (\
     klass, gabble_tubes_channel_##x##suffix)
