@@ -25,16 +25,19 @@
 
 #include <telepathy-glib/base-connection.h>
 
-#include "gabble-muc-channel.h"
+#include "muc-channel.h"
 #include "bytestream-iface.h"
 
 G_BEGIN_DECLS
 
 typedef struct _GabbleTubesChannel GabbleTubesChannel;
+typedef struct _GabbleTubesChannelPrivate GabbleTubesChannelPrivate;
 typedef struct _GabbleTubesChannelClass GabbleTubesChannelClass;
 
 struct _GabbleTubesChannelClass {
     GObjectClass parent_class;
+
+    TpDBusPropertiesMixinClass dbus_props_class;
 };
 
 struct _GabbleTubesChannel {
@@ -42,7 +45,7 @@ struct _GabbleTubesChannel {
 
     GabbleMucChannel *muc;
 
-    gpointer priv;
+    GabbleTubesChannelPrivate *priv;
 };
 
 GType gabble_tubes_channel_get_type (void);
