@@ -21,24 +21,22 @@
  */
 
 #include "config.h"
-
 #include "util.h"
 
-#include <glib.h>
 #include <stdio.h>
-#include <string.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <telepathy-glib/handle-repo-dynamic.h>
 
-#include "conn-aliasing.h"
-#include "sha1/sha1.h"
-#include "namespaces.h"
-#include "gabble-connection.h"
-#include "base64.h"
-
 #define DEBUG_FLAG GABBLE_DEBUG_JID
+
+#include "base64.h"
+#include "conn-aliasing.h"
+#include "connection.h"
 #include "debug.h"
+#include "namespaces.h"
+#include "sha1/sha1.h"
 
 gchar *
 sha1_hex (const gchar *bytes, guint len)
@@ -87,7 +85,7 @@ lm_message_node_add_own_nick (LmMessageNode *node,
 {
   gchar *nick;
   GabbleConnectionAliasSource source;
-  TpBaseConnection *base = (TpBaseConnection *)connection;
+  TpBaseConnection *base = (TpBaseConnection *) connection;
 
   source = _gabble_connection_get_cached_alias (connection,
         base->self_handle, &nick);
@@ -710,7 +708,7 @@ lm_message_node_extract_properties (LmMessageNode *node,
             }
           else
             {
-              g_debug ("invalid boolean value: %s", value);
+              DEBUG ("invalid boolean value: %s", value);
               continue;
             }
 
@@ -764,7 +762,7 @@ set_child_from_property (gpointer key,
   else
     {
       /* a type we don't know how to handle: ignore it */
-      g_debug ("property with unknown type \"%s\"",
+      DEBUG ("property with unknown type \"%s\"",
           g_type_name (G_VALUE_TYPE (gvalue)));
       return;
     }
