@@ -19,24 +19,24 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "vcard-manager.h"
 
 #include <string.h>
-#include <time.h>
 
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/heap.h>
 
+#define DEBUG_FLAG GABBLE_DEBUG_VCARD
+
 #include "base64.h"
 #include "conn-aliasing.h"
-#include "gabble-connection.h"
+#include "connection.h"
+#include "debug.h"
 #include "namespaces.h"
 #include "request-pipeline.h"
 #include "util.h"
 
-#define DEBUG_FLAG GABBLE_DEBUG_VCARD
-#include "debug.h"
-
 #define DEFAULT_REQUEST_TIMEOUT 60000
 #define VCARD_CACHE_ENTRY_TTL 60
 
@@ -63,7 +63,6 @@ enum
 G_DEFINE_TYPE(GabbleVCardManager, gabble_vcard_manager, G_TYPE_OBJECT);
 
 typedef struct _GabbleVCardCacheEntry GabbleVCardCacheEntry;
-typedef struct _GabbleVCardManagerPrivate GabbleVCardManagerPrivate;
 struct _GabbleVCardManagerPrivate
 {
   gboolean dispose_has_run;
@@ -72,7 +71,7 @@ struct _GabbleVCardManagerPrivate
   /* TpHandle borrowed from the entry => owned (GabbleVCardCacheEntry *) */
   GHashTable *cache;
 
-  /* Those (GabbleVCardCacheEntry *)s that have not expired, ordered by
+  /* Those (GabbleVCardCacheEntry *) s that have not expired, ordered by
    * increasing expiry time; borrowed from @cache */
   TpHeap *timed_cache;
 
@@ -172,8 +171,7 @@ gabble_vcard_manager_cache_quark (void)
   return quark;
 }
 
-#define GABBLE_VCARD_MANAGER_GET_PRIVATE(o)\
-  ((GabbleVCardManagerPrivate*)((o)->priv))
+#define GABBLE_VCARD_MANAGER_GET_PRIVATE(o) ((o)->priv)
 
 static void cache_entry_free (void *data);
 static gint cache_entry_compare (gconstpointer a, gconstpointer b);
@@ -331,7 +329,7 @@ cache_entry_get (GabbleVCardManager *manager, TpHandle handle)
 {
   GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->connection, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
   GabbleVCardCacheEntry *entry;
 
   entry = g_hash_table_lookup (priv->cache, GUINT_TO_POINTER (handle));
@@ -429,7 +427,7 @@ gabble_vcard_manager_invalidate_cache (GabbleVCardManager *manager,
   GabbleVCardCacheEntry *entry = g_hash_table_lookup (priv->cache,
       GUINT_TO_POINTER (handle));
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->connection, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
 
   g_return_if_fail (tp_handle_is_valid (contact_repo, handle, NULL));
 
@@ -453,12 +451,18 @@ static void complete_one_request (GabbleVCardManagerRequest *request,
 static void
 cache_entry_complete_requests (GabbleVCardCacheEntry *entry, GError *error)
 {
-  while (entry->pending_requests)
+  GSList *cur, *tmp;
+
+  tmp = g_slist_copy (entry->pending_requests);
+
+  for (cur = tmp; cur != NULL; cur = cur->next)
     {
-      GabbleVCardManagerRequest *request = entry->pending_requests->data;
+      GabbleVCardManagerRequest *request = cur->data;
 
       complete_one_request (request, error ? NULL : entry->vcard_node, error);
     }
+
+  g_slist_free (tmp);
 }
 
 static void
@@ -582,7 +586,7 @@ initial_request_cb (GabbleVCardManager *self,
                     gpointer user_data)
 {
   GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
-  gchar *alias = (gchar *)user_data;
+  gchar *alias = (gchar *) user_data;
   gchar *sha1;
 
   if (vcard)
@@ -614,7 +618,7 @@ status_changed_cb (GObject *object,
   GabbleVCardManager *self = GABBLE_VCARD_MANAGER (user_data);
   GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
   GabbleConnection *conn = GABBLE_CONNECTION (object);
-  TpBaseConnection *base = (TpBaseConnection *)conn;
+  TpBaseConnection *base = (TpBaseConnection *) conn;
 
   if (status == TP_CONNECTION_STATUS_CONNECTED)
     {
@@ -708,7 +712,7 @@ delete_request (GabbleVCardManagerRequest *request)
 static gboolean
 timeout_request (gpointer data)
 {
-  GabbleVCardManagerRequest *request = (GabbleVCardManagerRequest*) data;
+  GabbleVCardManagerRequest *request = (GabbleVCardManagerRequest *) data;
   GError err = { GABBLE_VCARD_MANAGER_ERROR,
       GABBLE_VCARD_MANAGER_ERROR_TIMEOUT, "Request timed out" };
 
@@ -771,7 +775,7 @@ observe_vcard (GabbleConnection *conn,
                LmMessageNode *vcard_node)
 {
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)conn, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) conn, TP_HANDLE_TYPE_CONTACT);
   const gchar *field = "<NICKNAME>";
   gchar *alias;
   const gchar *old_alias;
@@ -1203,7 +1207,7 @@ gabble_vcard_manager_request (GabbleVCardManager *self,
                               GObject *object)
 {
   GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
-  TpBaseConnection *connection = (TpBaseConnection *)priv->connection;
+  TpBaseConnection *connection = (TpBaseConnection *) priv->connection;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       connection, TP_HANDLE_TYPE_CONTACT);
   GabbleVCardManagerRequest *request;
@@ -1247,7 +1251,7 @@ gabble_vcard_manager_edit (GabbleVCardManager *self,
   va_list ap;
   size_t i;
   GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
-  TpBaseConnection *base = (TpBaseConnection *)priv->connection;
+  TpBaseConnection *base = (TpBaseConnection *) priv->connection;
   GabbleVCardManagerEditRequest *req;
   GabbleVCardCacheEntry *entry;
 
@@ -1375,7 +1379,7 @@ gabble_vcard_manager_get_cached (GabbleVCardManager *self,
   GabbleVCardCacheEntry *entry = g_hash_table_lookup (priv->cache,
       GUINT_TO_POINTER (handle));
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->connection, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
 
   g_return_val_if_fail (tp_handle_is_valid (contact_repo, handle, NULL),
       FALSE);
@@ -1405,7 +1409,7 @@ gabble_vcard_manager_get_cached_alias (GabbleVCardManager *manager,
 
   priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
   contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->connection, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
 
   g_return_val_if_fail (tp_handle_is_valid (contact_repo, handle, NULL), NULL);
 
@@ -1433,7 +1437,7 @@ gabble_vcard_manager_has_cached_alias (GabbleVCardManager *manager,
 
   priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
   contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *)priv->connection, TP_HANDLE_TYPE_CONTACT);
+      (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
 
   g_return_val_if_fail (tp_handle_is_valid (contact_repo, handle, NULL),
       FALSE);
