@@ -33,6 +33,14 @@ def make_result_iq(stream, iq):
 def acknowledge_iq(stream, iq):
     stream.send(make_result_iq(stream, iq))
 
+def sync_stream(q, stream):
+    """Used to ensure that Gabble has processed all stanzas sent to it."""
+
+    iq = IQ(stream, "get")
+    iq.addElement(('http://jabber.org/protocol/disco#info', 'query'))
+    stream.send(iq)
+    q.expect('stream-iq', query_ns='http://jabber.org/protocol/disco#info')
+
 class JabberAuthenticator(xmlstream.Authenticator):
     "Trivial XML stream authenticator that accepts one username/digest pair."
 
@@ -77,6 +85,7 @@ class JabberAuthenticator(xmlstream.Authenticator):
         self.xmlstream.send(result)
         self.xmlstream.dispatch(self.xmlstream, xmlstream.STREAM_AUTHD_EVENT)
 
+
 class XmppAuthenticator(xmlstream.Authenticator):
     def __init__(self, username, password):
         xmlstream.Authenticator.__init__(self)
@@ -84,7 +93,10 @@ class XmppAuthenticator(xmlstream.Authenticator):
         self.password = password
         self.authenticated = False
 
-    def streamStarted(self):
+    def streamStarted(self, root=None):
+        if root:
+            self.xmlstream.sid = root.getAttribute('id')
+
         self.xmlstream.sendHeader()
 
         if self.authenticated:
@@ -196,7 +208,7 @@ class JabberXmlStream(BaseXmlStream):
 class XmppXmlStream(BaseXmlStream):
     version = (1, 0)
 
-def prepare_test(event_func, params=None, authenticator=None, protocol=None):
+def make_connection(bus, event_func, params=None):
     default_params = {
         'account': 'test@localhost/Resource',
         'password': 'pass',
@@ -208,9 +220,10 @@ def prepare_test(event_func, params=None, authenticator=None, protocol=None):
     if params:
         default_params.update(params)
 
-    bus, conn = servicetest.prepare_test(event_func, 'gabble', 'jabber',
+    return servicetest.make_connection(bus, event_func, 'gabble', 'jabber',
         default_params)
 
+def make_stream(event_func, authenticator=None, protocol=None, port=4242):
     # set up Jabber server
 
     if authenticator is None:
@@ -222,16 +235,17 @@ def prepare_test(event_func, params=None, authenticator=None, protocol=None):
     stream = protocol(event_func, authenticator)
     factory = twisted.internet.protocol.Factory()
     factory.protocol = lambda *args: stream
-    reactor.listenTCP(4242, factory)
-    return bus, conn, stream
+    port = reactor.listenTCP(port, factory)
+    return (stream, port)
 
 def go(params=None, authenticator=None, protocol=None, start=None):
     # hack to ease debugging
     domish.Element.__repr__ = domish.Element.toXml
 
+    bus = dbus.SessionBus()
     handler = servicetest.EventTest()
-    bus, conn, stream = \
-        prepare_test(handler.handle_event, params, authenticator, protocol)
+    conn = make_connection(bus, handler.handle_event, params)
+    (stream, _) = make_stream(handler.handle_event, authenticator, protocol)
     handler.data = {
         'bus': bus,
         'conn': conn,
@@ -252,50 +266,76 @@ def go(params=None, authenticator=None, protocol=None, start=None):
 
     reactor.run()
 
-def exec_test(fun, params=None, protocol=None, timeout=None):
-    queue = servicetest.IteratingEventQueue(timeout)
+def install_colourer():
+    def red(s):
+        return '\x1b[31m%s\x1b[0m' % s
 
-    queue.verbose = (os.environ.get('CHECK_TWISTED_VERBOSE', '') != '')
-    if '-v' in sys.argv:
-        queue.verbose = True
+    def green(s):
+        return '\x1b[32m%s\x1b[0m' % s
 
-    bus, conn, stream = prepare_test(queue.append, params, protocol=protocol)
+    patterns = {
+        'handled': green,
+        'not handled': red,
+        }
 
+    class Colourer:
+        def __init__(self, fh, patterns):
+            self.fh = fh
+            self.patterns = patterns
+
+        def write(self, s):
+            f = self.patterns.get(s, lambda x: x)
+            self.fh.write(f(s))
+
+    sys.stdout = Colourer(sys.stdout, patterns)
+    return sys.stdout
+
+
+def exec_test_deferred (fun, params, protocol=None, timeout=None):
     # hack to ease debugging
     domish.Element.__repr__ = domish.Element.toXml
+    colourer = None
 
     if sys.stdout.isatty():
-        def red(s):
-            return '\x1b[31m%s\x1b[0m' % s
+        colourer = install_colourer()
 
-        def green(s):
-            return '\x1b[32m%s\x1b[0m' % s
+    queue = servicetest.IteratingEventQueue(timeout)
+    queue.verbose = (
+        os.environ.get('CHECK_TWISTED_VERBOSE', '') != ''
+        or '-v' in sys.argv)
 
-        patterns = {
-            'handled': green,
-            'not handled': red,
-            }
+    bus = dbus.SessionBus()
+    conn = make_connection(bus, queue.append, params)
+    (stream, port) = make_stream(queue.append, protocol=protocol)
 
-        class Colourer:
-            def __init__(self, fh, patterns):
-                self.fh = fh
-                self.patterns = patterns
-
-            def write(self, s):
-                f = self.patterns.get(s, lambda x: x)
-                self.fh.write(f(s))
-
-        sys.stdout = Colourer(sys.stdout, patterns)
+    error = None
 
     try:
         fun(queue, bus, conn, stream)
-    finally:
-        try:
-            conn.Disconnect()
-            # second call destroys object
-            conn.Disconnect()
-        except dbus.DBusException, e:
-            pass
+    except Exception, e:
+        import traceback
+        traceback.print_exc()
+        error = e
+
+    try:
+        if colourer:
+          sys.stdout = colourer.fh
+        d = port.stopListening()
+        if error is None:
+            d.addBoth((lambda *args: reactor.crash()))
+        else:
+            # please ignore the POSIX behind the curtain
+            d.addBoth((lambda *args: os._exit(1)))
+
+        conn.Disconnect()
+        # second call destroys object
+        conn.Disconnect()
+    except dbus.DBusException, e:
+        pass
+
+def exec_test(fun, params=None, protocol=None, timeout=None):
+  reactor.callWhenRunning (exec_test_deferred, fun, params, protocol, timeout)
+  reactor.run()
 
 # Useful routines for server-side vCard handling
 current_vcard = domish.Element(('vcard-temp', 'vCard'))
