@@ -18,7 +18,7 @@ def _expect_contact_list_channel(q, bus, conn, name, contacts):
     assert handle_type == HT_CONTACT_LIST, handle_type
     inspected = conn.InspectHandles(handle_type, [handle])[0]
     assert inspected == name, (inspected, name)
-    chan = bus.get_object(conn._named_service, path)
+    chan = bus.get_object(conn.bus_name, path)
     group_iface = dbus.Interface(chan,
         u'org.freedesktop.Telepathy.Channel.Interface.Group')
     inspected = conn.InspectHandles(1, group_iface.GetMembers())
@@ -31,12 +31,24 @@ def _expect_group_channel(q, bus, conn, name, contacts):
     assert handle_type == HT_GROUP, handle_type
     inspected = conn.InspectHandles(handle_type, [handle])[0]
     assert inspected == name, (inspected, name)
-    chan = bus.get_object(conn._named_service, path)
+    chan = bus.get_object(conn.bus_name, path)
     group_iface = dbus.Interface(chan,
         u'org.freedesktop.Telepathy.Channel.Interface.Group')
     inspected = conn.InspectHandles(1, group_iface.GetMembers())
     assert inspected == contacts, (inspected, contacts)
 
+    channel_props = chan.GetAll(
+            'org.freedesktop.Telepathy.Channel',
+            dbus_interface='org.freedesktop.DBus.Properties')
+    assert channel_props['TargetID'] == name, channel_props
+
+    future_props = chan.GetAll(
+            'org.freedesktop.Telepathy.Channel.FUTURE',
+            dbus_interface='org.freedesktop.DBus.Properties')
+    assert future_props['Requested'] == False
+    assert future_props['InitiatorID'] == ''
+    assert future_props['InitiatorHandle'] == 0
+
 def test(q, bus, conn, stream):
     conn.Connect()
     # q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
